/*===========================================================================*/
/*              This supportpac is maintained by Wayne M. Schutz             */
/*                                                                           */
/*              Chris Petty is the original author.                          */
/*                                                                           */
/*  (C) Copyright IBM Corporation 1995, 1997, 1998, 1999, 2000, 2002, 2003   */
/*===========================================================================*/
/*                                                                           */
/*  See the read.me file for change history                                  */
/*                                                                           */
/*===========================================================================*/
/* Module Name: authinfo.c                                                   */
/*===========================================================================*/

/* Include standard libraries */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

/* Include MQSeries headers */
#include <cmqc.h>
#include <cmqcfc.h>
#include <cmqxc.h>


/* Include Application headers */
#include "saveqmgr.h"

/* If this isn't v5.3, don't bother compiling anything */
#if defined (MQ_AUTH_INFO_NAME_LENGTH)

/* ------------------------------------------------------------------------ */
/* ProcessNamelistStringParm()                                              */
/* ProcessNamelistIntegerParm()                                             */
/*                                                                          */
/* These two processes are given a a block of storage that                  */
/* contains the definition of a single attribute relating to an individual  */
/* authinfo. The first handles string parameters, the second integer.       */
/* The calling process has already decided which type of parameter this is  */
/* and is responsible for managing the storage. These procedures decide     */
/* which of the possible attributes this one is and extract the value.      */
/*                                                                          */
/* Follows is a list of all attributes possible for a process definition.   */
/*                                                                          */
/* The AuthInfoDefn structure contains entries for each of the attributes.  */
/* ------------------------------------------------------------------------ */

extern int noisy;

void ProcessAuthInfoStringParm( MQCFST    *pPCFString
                          , AuthInfoParms *pAuthInfoDefn
                          , MQLONG       *iCC
                          )
{
   /* Initialise return codes */
   *iCC = OK;

   switch ( pPCFString->Parameter ) {
   case MQCA_AUTH_INFO_NAME:
      memset( pAuthInfoDefn->AuthInfoName
            , ' ', MQ_AUTH_INFO_NAME_LENGTH );
      memcpy( pAuthInfoDefn->AuthInfoName
            , pPCFString->String, pPCFString->StringLength );
      break;
   case MQCA_AUTH_INFO_DESC:
      memset( pAuthInfoDefn->AuthInfoDesc
            , ' ', MQ_AUTH_INFO_DESC_LENGTH );
      memcpy( pAuthInfoDefn->AuthInfoDesc
            , pPCFString->String, pPCFString->StringLength );
      break;
   case MQCA_ALTERATION_DATE:
      memset( pAuthInfoDefn->AlterationDate
            , ' ', MQ_CREATION_DATE_LENGTH );
      memcpy( pAuthInfoDefn->AlterationDate
            , pPCFString->String, pPCFString->StringLength);
      break;
   case MQCA_ALTERATION_TIME:
      memset( pAuthInfoDefn->AlterationTime
            , ' ', MQ_CREATION_TIME_LENGTH );
      memcpy( pAuthInfoDefn->AlterationTime
            , pPCFString->String, pPCFString->StringLength);
      break;
   case MQCA_AUTH_INFO_CONN_NAME:
      memset( pAuthInfoDefn->ConnName
            , ' ', MQ_CONN_NAME_LENGTH );
      memcpy( pAuthInfoDefn->ConnName
            , pPCFString->String, pPCFString->StringLength);
      break;
   case MQCA_LDAP_PASSWORD:
      memset( pAuthInfoDefn->Password
            , ' ', MQ_LDAP_PASSWORD_LENGTH );
      memcpy( pAuthInfoDefn->Password
            , pPCFString->String, pPCFString->StringLength);
      break;
   case MQCA_LDAP_USER_NAME:
      memset( pAuthInfoDefn->UserName
            , ' ', sizeof(pAuthInfoDefn->UserName) );
      memcpy( pAuthInfoDefn->UserName
            , pPCFString->String, pPCFString->StringLength);
      break;
   default:
       if(noisy) fprintf(stderr, "Unknown PCF String Parameter: %ld\n",
              pPCFString->Parameter);
      *iCC = WARNING;
      break;
   } /* endswitch */

   return;
}


/*************************************************************/
void ProcessAuthInfoIntegerParm( MQCFIN    *pPCFInteger
                          , AuthInfoParms *pAuthInfoDefn
                           , MQLONG       *iCC
                           )
{
   /* Initialise return codes */
   *iCC = OK;


   switch ( pPCFInteger->Parameter ) {
     case MQIA_AUTH_INFO_TYPE:
        pAuthInfoDefn->AuthInfoType = pPCFInteger->Value;
        break;
   default:
       if(noisy) fprintf(stderr, "Unknown PCF Integer Parameter: %ld\n",
              pPCFInteger->Parameter);
      *iCC = WARNING;
      break;
   } /* endswitch */

   return;
}



/* ------------------------------------------------------------------------ */
/*                                                                          */
/* This process takes the attributes of a single authinfo and adds them     */
/* to the end of a file, SAVEQMGR.TST, which can be found in the current    */
/* directory.                                                               */
/*                                                                          */
/* The file is of a format suitable for subsequent input to RUNMQSC.        */
/*                                                                          */
/* ------------------------------------------------------------------------ */
void AddToFileAUTHINFO(
                       AuthInfoParms *pAuthInfoDefn,
                       int UserVersion,
                       char * fileName,   /* v32 Name of the output file  */
                        FILE * fp,       /* v51 */
                       MQLONG *iCC )
{
   char *  ParmBuffer;   /* Temporary buffer to hold for output to file */

   if ( !( ParmBuffer = (char *) malloc (PBSIZE) )) {
     fprintf (stderr, "Failed to allocate %ld bytes \n", PBSIZE);
     exit(2);
   }

   /* Initialise return codes */
   *iCC = OK;

   /* Append these details to the end of the current output file (v32) */

      MQsprintf( ParmBuffer
               , "DEFINE AUTHINFO('%s') +\n"
               , pAuthInfoDefn->AuthInfoName
               , MQ_AUTH_INFO_NAME_LENGTH
               );
      fputs( ParmBuffer, fp );

      MQsprintf( ParmBuffer, "* ALTDATE (%s) + \n",
               pAuthInfoDefn->AlterationDate,
               MQ_CREATION_DATE_LENGTH);
      fputs( ParmBuffer, fp);

      MQsprintf( ParmBuffer, "* ALTTIME (%s) + \n",
               pAuthInfoDefn->AlterationTime,
               MQ_CREATION_TIME_LENGTH);
      fputs( ParmBuffer, fp);

      /* For now, this suffices.  If More AuthTypes are introduced */
      /* We'll have to get more sophisicated                       */
      if (pAuthInfoDefn->AuthInfoType)
        {
          sprintf( ParmBuffer, "       AUTHTYPE(CRLLDAP) +\n" );
        }
      fputs( ParmBuffer, fp );

      MQsprintf( ParmBuffer
               ,           "       DESCR('%s') +\n"
               , pAuthInfoDefn->AuthInfoDesc
               , MQ_AUTH_INFO_DESC_LENGTH
               );
      fputs( ParmBuffer, fp );

      MQsprintf( ParmBuffer
               , "       CONNAME('%s') +\n"
               , (char*) pAuthInfoDefn->ConnName
               , MQ_AUTH_INFO_CONN_NAME_LENGTH
               );
      fputs( ParmBuffer, fp );

      MQsprintf( ParmBuffer
               , "**     LDAPPWD('%s') +\n"
               , (char*) pAuthInfoDefn->Password
               , MQ_LDAP_PASSWORD_LENGTH
               );
      fputs( ParmBuffer, fp );

      MQsprintf( ParmBuffer
               , "       LDAPUSER('%s') +\n"
               , (char*) pAuthInfoDefn->UserName
               , sizeof(pAuthInfoDefn->UserName)
               );
      fputs( ParmBuffer, fp );

      sprintf( ParmBuffer, "        REPLACE\n\n" );
      fputs( ParmBuffer, fp );

   free (ParmBuffer);
   return;


}

/* end of don't compile anything */
#endif
