/*===========================================================================*/
/*              This supportpac is maintained by Wayne M. Schutz             */
/*                                                                           */
/*              Chris Petty is the original author.                          */
/*                                                                           */
/*  (C) Copyright IBM Corporation 1995, 1997, 1998, 1999, 2000, 2002, 2003   */
/*===========================================================================*/
/*                                                                           */
/*  See the read.me file for change history                                  */
/*                                                                           */
/*===========================================================================*/
/* Module Name: CHANNEL.C                                                    */
/*===========================================================================*/

/* Include standard libraries */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* Include MQSeries headers */
#include <cmqc.h>
#include <cmqcfc.h>
#include <cmqxc.h>

/* Include Application headers */
#include "saveqmgr.h"



/* ------------------------------------------------------------------------ */
/* ProcessChannelStringParm()                                               */
/* ProcessChannelIntegerParm()                                              */
/*                                                                          */
/* These two processes are given a pointer to a block of storage that       */
/* contains the definition of a single attribute relating to an individual  */
/* channel. The first handles string parameters, the second integer.        */
/* The calling process has already decided which type of parameter this is  */
/* and is responsible for managing the storage. These procedures decide     */
/* which of the possible attributes this one is and extract the value.      */
/*                                                                          */
/* Follows is a list of all attributes possible for the complete            */
/* set of channel types.                                                    */
/*                                                                          */
/* The parameters are not all relevant for all queue types.                 */
/*                                                                          */
/* The ChlDefn structure contains entries for each of the attributes,       */
/* only the attributes relevant for this channel type will be meaningful.   */
/* When we later write the file we will ensure that, based on channel type, */
/* only the relevant fields are used.                                       */
/* ------------------------------------------------------------------------ */
extern int noisy;
extern int chlReset;

void ProcessChannelStringParm( MQCFST   *pPCFString
                             , ChlParms *ChlDefn
                             , MQLONG      *iCC
                             )
{
   /* Initialise return codes */
   *iCC = OK;

   switch ( pPCFString->Parameter ) {
   case MQCACH_CHANNEL_NAME:
      memset( ChlDefn->ChannelName
               , ' ', MQ_CHANNEL_NAME_LENGTH );
      memcpy( ChlDefn->ChannelName
               , pPCFString->String, pPCFString -> StringLength);
      break;
   case MQCACH_MODE_NAME:
      memset( ChlDefn->ModeName
               , ' ', MQ_MODE_NAME_LENGTH );
      memcpy( ChlDefn->ModeName
               , pPCFString->String, pPCFString -> StringLength);
      break;
   case MQCACH_TP_NAME:
      memset( ChlDefn->TpName
               , ' ', MQ_TP_NAME_LENGTH );
      memcpy( ChlDefn->TpName
               , pPCFString->String, pPCFString -> StringLength);
      break;
   case MQCA_Q_MGR_NAME:
      memset( ChlDefn->QMgrName
               , ' ', MQ_Q_MGR_NAME_LENGTH );
      memcpy( ChlDefn->QMgrName
               , pPCFString->String, pPCFString -> StringLength);
      break;
   case MQCACH_XMIT_Q_NAME:
      memset( ChlDefn->XmitQName
               , ' ', MQ_Q_NAME_LENGTH );
      memcpy( ChlDefn->XmitQName
               , pPCFString->String, pPCFString -> StringLength);
      break;
   case MQCACH_CONNECTION_NAME:
      memset( ChlDefn->ConnectionName
               , ' ', MQ_CONN_NAME_LENGTH );
      memcpy( ChlDefn->ConnectionName
               , pPCFString->String, pPCFString -> StringLength);
      break;
   /* note that although we capture this attribute, it currently can only */
   /* have a value of blanks, so we never bother to generate a keyword */
   case MQCACH_MCA_NAME:
         memset( ChlDefn->MCAName
                  , ' ', MQ_MCA_NAME_LENGTH );
         memcpy( ChlDefn->MCAName
                  , pPCFString->String, pPCFString -> StringLength);
      break;
   case MQCACH_DESC:
      memset( ChlDefn->ChannelDesc
               , ' ', MQ_CHANNEL_DESC_LENGTH );
      memcpy( ChlDefn->ChannelDesc
               , pPCFString->String, pPCFString -> StringLength);
      break;
   case MQCACH_SEC_EXIT_NAME:
      memset( ChlDefn->SecurityExit
               , ' ', MQ_EXIT_NAME_LENGTH );
      memcpy( ChlDefn->SecurityExit
               , pPCFString->String, pPCFString -> StringLength);
      break;
   case MQCACH_MSG_EXIT_NAME:
      memset( ChlDefn->MsgExit
               , ' ', MQ_TOTAL_EXIT_NAME_LENGTH );
      memcpy( ChlDefn->MsgExit
               , pPCFString->String, pPCFString -> StringLength);
      /* [v530] indicate type of string data */
      ChlDefn->MsgExitListFlag = 0;
      break;
   case MQCACH_SEND_EXIT_NAME:
      memset( ChlDefn->SendExit
               , ' ', MQ_TOTAL_EXIT_NAME_LENGTH );
      memcpy( ChlDefn->SendExit
               , pPCFString->String, pPCFString -> StringLength);
      /* [v530] indicate type of string data */
      ChlDefn->SendExitListFlag = 0;
      break;
   case MQCACH_RCV_EXIT_NAME:
      memset( ChlDefn->ReceiveExit
               , ' ', MQ_TOTAL_EXIT_NAME_LENGTH );
      memcpy( ChlDefn->ReceiveExit
               , pPCFString->String, pPCFString -> StringLength);
      /* [v530] indicate type of string data */
      ChlDefn->ReceiveExitListFlag = 0;
      break;
   case MQCACH_SEC_EXIT_USER_DATA:
      memset( ChlDefn->SecurityUserData
               , ' ', MQ_EXIT_DATA_LENGTH );
      memcpy( ChlDefn->SecurityUserData
               , pPCFString->String, pPCFString -> StringLength);
      break;
   case MQCACH_MSG_EXIT_USER_DATA:
      memset( ChlDefn->MsgUserData
               , ' ', MQ_TOTAL_EXIT_DATA_LENGTH );
      memcpy( ChlDefn->MsgUserData
               , pPCFString->String, pPCFString -> StringLength);
      /* [v530] indicate type of string data */
      ChlDefn->MsgUserDataListFlag = 0;
      break;
   case MQCACH_SEND_EXIT_USER_DATA:
      memset( ChlDefn->SendUserData
               , ' ', MQ_TOTAL_EXIT_DATA_LENGTH );
      memcpy( ChlDefn->SendUserData
               , pPCFString->String, pPCFString -> StringLength);
      /* [v530] indicate type of string data */
      ChlDefn->SendUserDataListFlag = 0;
      break;
   case MQCACH_RCV_EXIT_USER_DATA:
      memset( ChlDefn->ReceiveUserData
               , ' ', MQ_TOTAL_EXIT_DATA_LENGTH );
      memcpy( ChlDefn->ReceiveUserData
               , pPCFString->String, pPCFString -> StringLength);
      /* [v530] indicate type of string data */
      ChlDefn->ReceiveUserDataListFlag = 0;
      break;
   case MQCACH_MCA_USER_ID:
      memset( ChlDefn->MCAUserIdentifier
               , ' ', XZMCAUSERLEN );
      memcpy( ChlDefn->MCAUserIdentifier
               , pPCFString->String, pPCFString -> StringLength);
      break;
   case MQCACH_USER_ID:
      memset( ChlDefn->UserIdentifier
               , ' ', MQ_USER_ID_LENGTH );
      memcpy( ChlDefn->UserIdentifier
               , pPCFString->String, pPCFString -> StringLength);
      break;
   case MQCACH_PASSWORD:
      memset( ChlDefn->Password
               , ' ', MQ_PASSWORD_LENGTH );
      memcpy( ChlDefn->Password
               , pPCFString->String, pPCFString -> StringLength);
      break;
   /* [wms] start msg retry attributes (was in V2) */
   case MQCACH_MR_EXIT_NAME:

      memset( ChlDefn->MsgRetryExit
               , ' ', MQ_EXIT_NAME_LENGTH );
      memcpy( ChlDefn->MsgRetryExit
               , pPCFString->String, pPCFString -> StringLength);
      ChlDefn->MsgRetryExitListFlag = 0;

      break;
   case MQCACH_MR_EXIT_USER_DATA:
      memset( ChlDefn->MsgRetryUserData
               , ' ', MQ_EXIT_DATA_LENGTH );
      memcpy( ChlDefn->MsgRetryUserData
               , pPCFString->String, pPCFString -> StringLength);
      ChlDefn->MsgRetryUserDataListFlag = 0;
      break;
   /* [wms]  end msg retry attributes */

   /* [v51] begin v51 attributes */
   #if defined (MQCA_CLUSTER_NAME)
   case MQCA_CLUSTER_NAME:
      memset( ChlDefn->ClusterName
            , ' ', MQ_CLUSTER_NAME_LENGTH );
      memcpy( ChlDefn->ClusterName
            , pPCFString->String, pPCFString->StringLength);
      break;
   #endif
   #if defined (MQCA_CLUSTER_NAMELIST)
   case MQCA_CLUSTER_NAMELIST:
      memset( ChlDefn->ClusterNameList
            , ' ', MQ_OBJECT_NAME_LENGTH );
      memcpy( ChlDefn->ClusterNameList
            , pPCFString->String, pPCFString->StringLength);
      break;
   #endif
   #if defined (MQCA_ALTERATION_DATE)
   case MQCA_ALTERATION_DATE:
      memset( ChlDefn->AlterationDate
            , ' ', MQ_CREATION_DATE_LENGTH );
      memcpy( ChlDefn->AlterationDate
            , pPCFString->String, pPCFString->StringLength);
      break;
   case MQCA_ALTERATION_TIME:
      memset( ChlDefn->AlterationTime
            , ' ', MQ_CREATION_TIME_LENGTH );
      memcpy( ChlDefn->AlterationTime
            , pPCFString->String, pPCFString->StringLength);
      break;
   #endif
   /* [v51] end V51 attributes */

   /* [v530] begin V5.3 attributes */
   #if defined (MQ_LOCAL_ADDRESS_LENGTH)

   case MQCACH_LOCAL_ADDRESS:
      memset( ChlDefn->LocalAddr
            , ' ', MQ_LOCAL_ADDRESS_LENGTH );
      memcpy( ChlDefn->LocalAddr
            , pPCFString->String, pPCFString->StringLength);
      break;

   case MQCACH_SSL_CIPHER_SPEC:
      memset( ChlDefn->SslCipher
            , ' ', MQ_SSL_CIPHER_SPEC_LENGTH );
      memcpy( ChlDefn->SslCipher
            , pPCFString->String, pPCFString->StringLength);
      break;

   case MQCACH_SSL_PEER_NAME:
      memset( ChlDefn->SslPeer
            , ' ', MQ_SSL_PEER_NAME_LENGTH );
      memcpy( ChlDefn->SslPeer
            , pPCFString->String, pPCFString->StringLength);
      break;

   #endif
   /* [v530] end V5.3 attributes */






   default:
       if(noisy) fprintf(stderr, "Unknown PCF String Parameter: %ld\n",
              pPCFString->Parameter);
      *iCC = WARNING;
      break;
   } /* endswitch */

   return;
}

/********************* [v512] process lists of exits and exit data ***************/
void ProcessChannelStringListParm( MQCFSL    *pPCFString
                          , ChlParms *ChlDefn
                          , MQLONG       *iCC
                          )
{
   /* Initialise return codes */
   *iCC = OK;

   switch ( pPCFString->Parameter ) {
   case MQCACH_MSG_EXIT_NAME:
       makeStringFromList( ChlDefn->MsgExit,
                          pPCFString->Strings,
                          pPCFString->StringLength,
                          pPCFString->Count);
      /* [v530] indicate type of string data */
      ChlDefn->MsgExitListFlag = 1;
   break;
   case MQCACH_SEND_EXIT_NAME:
       makeStringFromList( ChlDefn->SendExit,
                          pPCFString->Strings,
                          pPCFString -> StringLength,
                          pPCFString->Count);
      /* [v530] indicate type of string data */
      ChlDefn->SendExitListFlag = 1;
      break;
   case MQCACH_RCV_EXIT_NAME:
       makeStringFromList( ChlDefn->ReceiveExit,
                          pPCFString->Strings,
                          pPCFString -> StringLength,
                          pPCFString->Count);
      /* [v530] indicate type of string data */
      ChlDefn->ReceiveExitListFlag = 1;
      break;
   case MQCACH_MSG_EXIT_USER_DATA:
       makeStringFromList( ChlDefn->MsgUserData,
                          pPCFString->Strings,
                          pPCFString -> StringLength,
                          pPCFString->Count);
      /* [v530] indicate type of string data */
      ChlDefn->MsgUserDataListFlag = 1;
      break;
   case MQCACH_SEND_EXIT_USER_DATA:
       makeStringFromList( ChlDefn->SendUserData,
                          pPCFString->Strings,
                          pPCFString -> StringLength,
                          pPCFString->Count);
      /* [v530] indicate type of string data */
      ChlDefn->SendUserDataListFlag = 1;
      break;
   case MQCACH_RCV_EXIT_USER_DATA:
       makeStringFromList( ChlDefn->ReceiveUserData,
                          pPCFString->Strings,
                          pPCFString -> StringLength,
                          pPCFString->Count);
      /* [v530] indicate type of string data */
      ChlDefn->ReceiveUserDataListFlag = 1;
      break;
   case MQCACH_MR_EXIT_NAME:
       makeStringFromList( ChlDefn->MsgRetryExit,
                          pPCFString->Strings,
                          pPCFString -> StringLength,
                          pPCFString->Count);
      ChlDefn->MsgRetryExitListFlag = 1;
      break;
   case MQCACH_MR_EXIT_USER_DATA:
       makeStringFromList( ChlDefn->MsgRetryUserData,
                          pPCFString->Strings,
                          pPCFString -> StringLength,
                          pPCFString->Count);
      ChlDefn->MsgRetryUserDataListFlag = 1;
      break;

   default:

       if(noisy) fprintf(stderr, "Unknown PCF String List Parameter: %ld\n",
              pPCFString->Parameter);
      *iCC = WARNING;
      break;

   } /* endswitch */

   return;
}

void ProcessChannelIntegerParm( MQCFIN   *pPCFInteger
                              , ChlParms *ChlDefn
                              , MQLONG      *iCC
                              )
{
   /* Initialise  return codes */
   *iCC = OK;

   switch ( pPCFInteger->Parameter ) {
   case MQIACH_CHANNEL_TYPE:
      ChlDefn->ChannelType = pPCFInteger->Value;
      break;
   case MQIACH_XMIT_PROTOCOL_TYPE:
      ChlDefn->TransportType = pPCFInteger->Value;
      break;
   case MQIACH_BATCH_SIZE:
      ChlDefn->BatchSize = pPCFInteger->Value;
      break;
   case MQIACH_DISC_INTERVAL:
      ChlDefn->DiscInterval = pPCFInteger->Value;
      break;
   case MQIACH_SHORT_RETRY:
      ChlDefn->ShortRetryCount = pPCFInteger->Value;
      break;
   case MQIACH_SHORT_TIMER:
      ChlDefn->ShortRetryInterval = pPCFInteger->Value;
      break;
   case MQIACH_LONG_RETRY:
      ChlDefn->LongRetryCount = pPCFInteger->Value;
      break;
   case MQIACH_LONG_TIMER:
      ChlDefn->LongRetryInterval = pPCFInteger->Value;
      break;
   case MQIACH_DATA_CONVERSION:
      ChlDefn->DataConversion = pPCFInteger->Value;
      break;
   case MQIACH_PUT_AUTHORITY:
      ChlDefn->PutAuthority = pPCFInteger->Value;
      break;
   case MQIACH_SEQUENCE_NUMBER_WRAP:
      ChlDefn->SeqNumberWrap = pPCFInteger->Value;
      break;
   case MQIACH_MAX_MSG_LENGTH:
      ChlDefn->MaxMsgLength = pPCFInteger->Value;
      break;
   case MQIACH_MCA_TYPE:
      ChlDefn->MCAType = pPCFInteger->Value;
      break;
   /* [wms] start msg retry attributes (was in V2) */
   case MQIACH_MR_COUNT:
      ChlDefn->MsgRetryCount = pPCFInteger->Value;
      break;
   case MQIACH_MR_INTERVAL:
      ChlDefn->MsgRetryInterval = pPCFInteger->Value;
      break;
   /* end msg retry attributes */
   #if defined (MQIACH_BATCH_INTERVAL)
   /* [wms] begin v5 attributes */
   case MQIACH_BATCH_INTERVAL:
      ChlDefn->BatchInterval = pPCFInteger->Value;
      break;
   #endif
   #if defined (MQIACH_HB_INTERVAL)
   case MQIACH_HB_INTERVAL:
      ChlDefn->HeartbeatInterval = pPCFInteger->Value;
      break;
   #endif
   #if defined (MQIACH_NPM_SPEED)
   case MQIACH_NPM_SPEED:
      ChlDefn->NonPersistentMsgSpeed = pPCFInteger->Value;
      break;
   /* [wms] end v5 attributes */
   #endif
   /* [v51] start v51 attributes */
   #if defined (MQIACH_NETWORK_PRIORITY)
   case MQIACH_NETWORK_PRIORITY:
       ChlDefn->NetworkPriority = pPCFInteger->Value;
     break;
   #endif
   /* [v51] end v51 attributes */

     /* [v52] Tandem NSK attribute */
 #ifdef MQIACH_INHIBIT_AUTOSTART
     case MQIACH_INHIBIT_AUTOSTART:
         ChlDefn->InhibitAutostart = pPCFInteger->Value;
         break;
  #endif
   /* [v52] end v52 attributes */


   /* [v530] begin V5.3 attributes */
   #if defined (MQ_LOCAL_ADDRESS_LENGTH)

     case MQIACH_BATCH_HB:
         ChlDefn->BatchHB = pPCFInteger->Value;
         break;
     case MQIACH_SSL_CLIENT_AUTH:
         ChlDefn->SslCAuth = pPCFInteger->Value;

         break;
   /* [v531] csd01 on v5.3 adds KAINT to mqsc */
   #if defined (MQIACH_KEEP_ALIVE_INTERVAL)
     case MQIACH_KEEP_ALIVE_INTERVAL :
         ChlDefn->KAInt = pPCFInteger->Value;
         break;
	#endif


  #endif

   default:
      if(noisy) fprintf(stderr, "Unknown PCF Integer Parameter: %ld\n",
              pPCFInteger->Parameter);
      *iCC = WARNING;
      break;
   } /* endswitch */

   return;
}


/* ------------------------------------------------------------------------ */
/*                                                                          */
/* This process takes the attributes of a single channel and adds           */
/* them to the end of a file, SAVEQMGR.TST, which can be found in the       */
/* current directory.                                                       */
/*                                                                          */
/* The file is of a format suitable for subsequent input to RUNMQSC.        */
/*                                                                          */
/* ------------------------------------------------------------------------ */

void AddToFileCHANNEL(
                       ChlParms *pChlDefn,
                       int UserVersion,
                       char * fileName, /* v32 Name of the output file  */
                        FILE *fp,  /* v51 file handle */
                       MQLONG *iCC )
{
   char    * ParmBuffer;   /* Temporary buffer to hold for output to file */
   int     Options;        /* Option flag for MQsprintf_X */

   /* Initialise return codes */
   *iCC = OK;

   if ( !( ParmBuffer = (char *) malloc (PBSIZE) )) {
     fprintf (stderr, "Failed to allocate %d bytes \n", PBSIZE);
     exit(2);
   }

   /* v510 removes open from here ... fp = fopen( fileName, OPENMODE ); */



    /* ----------------------------------------------------------------- */
    /* Add ChannelName and Type to file                                  */
    /* If unrecognised type then ignore this defintition, return warning */
    /* ----------------------------------------------------------------- */

    switch ( pChlDefn->ChannelType ) {
    case MQCHT_SENDER:
       MQsprintf( ParmBuffer
                , "DEFINE CHANNEL ('%s') CHLTYPE(SDR) +\n"
                , pChlDefn->ChannelName
                , MQ_CHANNEL_NAME_LENGTH
                );
       break;
    case MQCHT_SERVER:
       MQsprintf( ParmBuffer
                , "DEFINE CHANNEL ('%s') CHLTYPE(SVR) +\n"
                , pChlDefn->ChannelName
                , MQ_CHANNEL_NAME_LENGTH
                );
       break;
    case MQCHT_RECEIVER:
       MQsprintf( ParmBuffer
                , "DEFINE CHANNEL ('%s') CHLTYPE(RCVR) +\n"
                , pChlDefn->ChannelName
                , MQ_CHANNEL_NAME_LENGTH
                );
       break;
    case MQCHT_REQUESTER:
       MQsprintf( ParmBuffer
                , "DEFINE CHANNEL ('%s') CHLTYPE(RQSTR) +\n"
                , pChlDefn->ChannelName
                , MQ_CHANNEL_NAME_LENGTH
                );
       break;
    case MQCHT_SVRCONN:
       if (UserVersion > MQCMDL_LEVEL_110) { /* v32 don't create for MQ/Win */
          MQsprintf( ParmBuffer
                   , "DEFINE CHANNEL ('%s') CHLTYPE(SVRCONN) +\n"
                   , pChlDefn->ChannelName
                   , MQ_CHANNEL_NAME_LENGTH
                   );
          break;
         } else {  /* v32 ... just get out ... */
            /* v51 no longer closes here ... fclose( fp ); */
            return;
         }/* v32 End if*/
    case MQCHT_CLNTCONN:
       MQsprintf( ParmBuffer
                , "DEFINE CHANNEL ('%s') CHLTYPE(CLNTCONN) +\n"
                , pChlDefn->ChannelName
                , MQ_CHANNEL_NAME_LENGTH
                );
       break;
    /* [v51] start v51 attributes */
    #if defined(MQCHT_CLUSRCVR)
    case MQCHT_CLUSRCVR:
       if (UserVersion>=MQCMDL_LEVEL_510) {
           MQsprintf( ParmBuffer
                    , "DEFINE CHANNEL ('%s') CHLTYPE(CLUSRCVR) +\n"
                    , pChlDefn->ChannelName
                    , MQ_CHANNEL_NAME_LENGTH
                    );
       } else {
         return;  /* user doesn't want V51 stuff */
       }/* End if*/
       break;
    #endif

    #if defined(MQCHT_CLUSSDR)
    case MQCHT_CLUSSDR:
       if (UserVersion>=MQCMDL_LEVEL_510) {
           MQsprintf( ParmBuffer
                    , "DEFINE CHANNEL ('%s') CHLTYPE(CLUSSDR) +\n"
                    , pChlDefn->ChannelName
                    , MQ_CHANNEL_NAME_LENGTH
                    );
       } else {
         return;  /* user doesn't want V51 stuff */
       }/* End if*/
       break;
    #endif
    /* [v51] end of V51 channel types */

    default:
       *iCC = WARNING;
       if(noisy) fprintf(stderr, "Unknown channel type = %ld \n",
              pChlDefn->ChannelType );
    } /* endswitch */

    fputs( ParmBuffer, fp );


      #if defined (MQCA_ALTERATION_DATE)
      MQsprintf( ParmBuffer, "* ALTDATE (%s) + \n",
               pChlDefn->AlterationDate,
               MQ_CREATION_DATE_LENGTH);
      fputs( ParmBuffer, fp);
      MQsprintf( ParmBuffer, "* ALTTIME (%s) + \n",
               pChlDefn->AlterationTime,
               MQ_CREATION_TIME_LENGTH);
      fputs( ParmBuffer, fp);
      #endif


    /* Since trptype = tcp is the only valid one                   */
    /* (and also the default, don't generate it if mq/win          */

    if (UserVersion > MQCMDL_LEVEL_110) {

     switch ( pChlDefn->TransportType ) {
     case MQXPT_LU62:
          sprintf( ParmBuffer, "       TRPTYPE(LU62) +\n" );
          fputs( ParmBuffer, fp );
          if (  pChlDefn->ChannelType != MQCHT_RECEIVER
             && pChlDefn->ChannelType != MQCHT_SVRCONN ) {
                      MQsprintf( ParmBuffer
                               , "       MODENAME('%s') +\n"
                               , pChlDefn->ModeName
                               , MQ_MODE_NAME_LENGTH
                               );
                      fputs( ParmBuffer, fp );
                      MQsprintf( ParmBuffer
                               , "       TPNAME('%s') +\n"
                               , pChlDefn->TpName
                               , MQ_TP_NAME_LENGTH
                               );
                      fputs( ParmBuffer, fp );
              }
        break;
     case MQXPT_TCP:
        sprintf( ParmBuffer, "       TRPTYPE(TCP) +\n" );
        fputs( ParmBuffer, fp );
        break;
     case MQXPT_NETBIOS:
        sprintf( ParmBuffer, "       TRPTYPE(NETBIOS) +\n" );
        fputs( ParmBuffer, fp );
        break;
     #if defined (MQXPT_SPX)
     case MQXPT_SPX:      /* [WMS] added for unix & dec */
        sprintf( ParmBuffer, "       TRPTYPE(SPX) +\n" );
        fputs( ParmBuffer, fp );
        break;
     #endif
     #if defined (MQXPT_DECNET)
     case MQXPT_DECNET:
        sprintf( ParmBuffer, "       TRPTYPE(DECNET) +\n" );
        fputs( ParmBuffer, fp );
        break;
     #endif
     #if defined (MQXPT_UDP)
     case MQXPT_UDP:
        sprintf( ParmBuffer, "       TRPTYPE(UPD) +\n" );
        fputs( ParmBuffer, fp );
        break;
     #endif
     } /* endswitch */
   }

    /* ------------------------------------------------------------- */
    /* [v51] Write out options in logical groups, based on cmd level  */
    /* requested by user and channel types.                          */
    /* ------------------------------------------------------------- */

      if (UserVersion>=MQCMDL_LEVEL_500 && (
           pChlDefn->ChannelType == MQCHT_SERVER
        || pChlDefn->ChannelType == MQCHT_SENDER
        #if defined (MQCHT_CLUSSDR)
        || pChlDefn->ChannelType == MQCHT_CLUSSDR
        || pChlDefn->ChannelType == MQCHT_CLUSRCVR
        #endif
       )) {
            sprintf( ParmBuffer, "       BATCHINT(%ld) +\n",
                     pChlDefn->BatchInterval );
            fputs( ParmBuffer, fp );

        /* [v530] Batchb for SDR, SVR, CLUSSDR and CLUSSVR */
        #if defined (MQIACH_BATCH_HB)
            if (UserVersion>=MQCMDL_LEVEL_530) {
                sprintf( ParmBuffer, "       BATCHHB(%ld) +\n",
                         pChlDefn->BatchHB );
                fputs( ParmBuffer, fp );
            }
        #endif
      }

      if (  pChlDefn->ChannelType == MQCHT_SENDER
         || pChlDefn->ChannelType == MQCHT_SERVER
         || pChlDefn->ChannelType == MQCHT_RECEIVER
         || pChlDefn->ChannelType == MQCHT_REQUESTER
        #if defined (MQCHT_CLUSSDR)
        || pChlDefn->ChannelType == MQCHT_CLUSSDR
        || pChlDefn->ChannelType == MQCHT_CLUSRCVR
        #endif
         ) {

             sprintf( ParmBuffer, "       BATCHSZ(%ld) +\n",
                      pChlDefn->BatchSize );
             fputs( ParmBuffer, fp );
      }

      #if defined (MQCHT_CLUSSDR)
      if (UserVersion>=MQCMDL_LEVEL_510 && (
           pChlDefn->ChannelType == MQCHT_CLUSSDR
        || pChlDefn->ChannelType == MQCHT_CLUSRCVR

       )) {

         MQsprintf( ParmBuffer
                  , "       CLUSTER('%s') +\n"
                  , pChlDefn->ClusterName
                  , MQ_CLUSTER_NAME_LENGTH
                  );
         fputs( ParmBuffer, fp );
         MQsprintf( ParmBuffer
                  , "       CLUSNL('%s') +\n"
                  , pChlDefn->ClusterNameList
                  , MQ_CLUSTER_NAME_LENGTH
                  );
         fputs( ParmBuffer, fp );

        }
      #endif

      if (  pChlDefn->ChannelType == MQCHT_SENDER
         || pChlDefn->ChannelType == MQCHT_REQUESTER
         || pChlDefn->ChannelType == MQCHT_CLNTCONN
         || pChlDefn->ChannelType == MQCHT_SERVER
         #if defined (MQCHT_CLUSSDR)
         || pChlDefn->ChannelType == MQCHT_CLUSSDR
         || pChlDefn->ChannelType == MQCHT_CLUSRCVR
         #endif
         ) {

         MQsprintf( ParmBuffer
                  , "       CONNAME('%s') +\n"
                  , pChlDefn->ConnectionName
                  , MQ_CONN_NAME_LENGTH
                  );
         fputs( ParmBuffer, fp );

         /* [v530] Locladdr for SDR, SVR, RQSTR, CLNTCONN, CLUSSDR, CLUSRCVR */
        #if defined (MQCACH_LOCAL_ADDRESS)
            if (UserVersion>=MQCMDL_LEVEL_530) {
                MQsprintf( ParmBuffer
                         , "       LOCLADDR('%s') +\n"
                         , pChlDefn->LocalAddr
                         , MQ_LOCAL_ADDRESS_LENGTH );
                fputs( ParmBuffer, fp );
            }
        #endif


         }



      if (UserVersion > MQCMDL_LEVEL_110 && (
            pChlDefn->ChannelType == MQCHT_SENDER
         || pChlDefn->ChannelType == MQCHT_SERVER
         #if defined (MQCHT_CLUSSDR)
         || pChlDefn->ChannelType == MQCHT_CLUSSDR
         || pChlDefn->ChannelType == MQCHT_CLUSRCVR
         #endif
            )) {

            if ( pChlDefn->DataConversion == MQCDC_NO_SENDER_CONVERSION )
               sprintf( ParmBuffer, "       CONVERT(NO) +\n" );
            else
               sprintf( ParmBuffer, "       CONVERT(YES) +\n" );
            fputs( ParmBuffer, fp );

         }

      MQsprintf( ParmBuffer
               , "       DESCR('%s') +\n"
               , pChlDefn->ChannelDesc
               , MQ_CHANNEL_DESC_LENGTH
               );
      fputs( ParmBuffer, fp );


      if (  pChlDefn->ChannelType == MQCHT_SENDER
         || pChlDefn->ChannelType == MQCHT_SERVER
         #if defined (MQCHT_CLUSSDR)
         || pChlDefn->ChannelType == MQCHT_CLUSSDR
         || pChlDefn->ChannelType == MQCHT_CLUSRCVR
         #endif
         ) {

         sprintf( ParmBuffer, "       DISCINT(%ld) +\n",
                  pChlDefn->DiscInterval );
         fputs( ParmBuffer, fp );
      }


      if (UserVersion>=MQCMDL_LEVEL_500)
      {
         sprintf( ParmBuffer, "       HBINT(%ld) +\n",
                  pChlDefn->HeartbeatInterval );
         fputs( ParmBuffer, fp );
      }

      if (  pChlDefn->ChannelType == MQCHT_SENDER
         || pChlDefn->ChannelType == MQCHT_SERVER
         #if defined (MQCHT_CLUSSDR)
         || pChlDefn->ChannelType == MQCHT_CLUSSDR
         || pChlDefn->ChannelType == MQCHT_CLUSRCVR
         #endif
         ) {

         sprintf( ParmBuffer, "       LONGRTY(%ld) +\n",
                  pChlDefn->LongRetryCount );
         fputs( ParmBuffer, fp );

         sprintf( ParmBuffer, "       LONGTMR(%ld) +\n",
                  pChlDefn->LongRetryInterval );
         fputs( ParmBuffer, fp );

         sprintf( ParmBuffer, "       SHORTRTY(%ld) +\n",
                  pChlDefn->ShortRetryCount );
         fputs( ParmBuffer, fp );

         sprintf( ParmBuffer, "       SHORTTMR(%ld) +\n",
                  pChlDefn->ShortRetryInterval );
         fputs( ParmBuffer, fp );

      }

      sprintf( ParmBuffer, "       MAXMSGL(%ld) +\n",
               pChlDefn->MaxMsgLength );
      fputs( ParmBuffer, fp );

      /* Note: MCANAME can only be set to blanks, so we don't bother to */
      /* write it to the output file .                                  */

      if ( (MQAT_DEFAULT != MQAT_OS400 &&       /* not the 400 platform */
           UserVersion > MQCMDL_LEVEL_110) && ( /* and not for MQ/win   */
            pChlDefn->ChannelType == MQCHT_SENDER
         || pChlDefn->ChannelType == MQCHT_SERVER
         || pChlDefn->ChannelType == MQCHT_REQUESTER
         #if defined (MQCHT_CLUSSDR)
         || pChlDefn->ChannelType == MQCHT_CLUSSDR
         #endif
         )) {

            if ( pChlDefn->MCAType == MQMCAT_PROCESS )
               sprintf( ParmBuffer, "       MCATYPE(PROCESS) +\n" );
            else
               sprintf( ParmBuffer, "       MCATYPE(THREAD) +\n" );
            fputs( ParmBuffer, fp );

         }

      if (  pChlDefn->ChannelType != MQCHT_CLNTCONN ) {

         MQsprintf( ParmBuffer
                  , "       MCAUSER('%s') +\n"
                  , pChlDefn->MCAUserIdentifier
                  , XZMCAUSERLEN
                  );
         fputs( ParmBuffer, fp );

      }

      if (UserVersion > MQCMDL_LEVEL_110 && (
            pChlDefn->ChannelType == MQCHT_RECEIVER
         || pChlDefn->ChannelType == MQCHT_REQUESTER
         #if defined (MQCHT_CLUSRCVR)
         || pChlDefn->ChannelType == MQCHT_CLUSRCVR
         #endif
         )) {

         /* [v530] Depending on whether we're dealing with a string */
         /* or chained exits, pass the correct flags to MQsprintf_X */
         Options = MQSPRINTF_X_LINE_BREAKS + MQSPRINTF_X_NO_TRAILING_BLANKS;

         if (! pChlDefn->MsgRetryUserDataListFlag ) {
              Options += MQSPRINTF_X_DOUBLE_QUOTES;
              MQsprintf_X( ParmBuffer
                            , "       MRDATA('%s') +\n"
                            , pChlDefn->MsgRetryUserData
                            , MQ_EXIT_DATA_LENGTH
                            , Options
                          );
         } else {
              MQsprintf_X( ParmBuffer
                            , "       MRDATA(%s) +\n"
                            , pChlDefn->MsgRetryUserData
                            , MQ_TOTAL_EXIT_DATA_LENGTH
                            , Options
                          );
         }
         fputs( ParmBuffer, fp );

         /* [v530] Depending on whether we're dealing with a string */
         /* or chained exits, pass the correct flags to MQsprintf_X */
         Options = MQSPRINTF_X_LINE_BREAKS + MQSPRINTF_X_NO_TRAILING_BLANKS;

         if (! pChlDefn->MsgRetryExitListFlag ) {
              Options += MQSPRINTF_X_DOUBLE_QUOTES;
              MQsprintf_X( ParmBuffer
                            , "       MREXIT('%s') +\n"
                            , pChlDefn->MsgRetryExit
                            , MQ_EXIT_NAME_LENGTH
                            , Options
                          );
         } else {
              MQsprintf_X( ParmBuffer
                            , "       MREXIT(%s) +\n"
                            , pChlDefn->MsgRetryExit
                            , MQ_TOTAL_EXIT_NAME_LENGTH
                            , Options
                          );
         }
         fputs( ParmBuffer, fp );

            sprintf( ParmBuffer, "       MRRTY(%ld) +\n",
                     pChlDefn->MsgRetryCount );
            fputs( ParmBuffer, fp );
            sprintf( ParmBuffer, "       MRTMR(%ld) +\n",
                     pChlDefn->MsgRetryInterval );
            fputs( ParmBuffer, fp );
      }

      if (  pChlDefn->ChannelType != MQCHT_CLNTCONN &&
            pChlDefn->ChannelType != MQCHT_SVRCONN
       ) {

         /* [v530] Depending on whether we're dealing with a string */
         /* or chained exits, pass the correct flags to MQsprintf_X */
         Options = MQSPRINTF_X_LINE_BREAKS + MQSPRINTF_X_NO_TRAILING_BLANKS;

         if (! pChlDefn->MsgUserDataListFlag ) {
              Options += MQSPRINTF_X_DOUBLE_QUOTES;
              MQsprintf_X( ParmBuffer
                            , "       MSGDATA('%s') +\n"
                            , pChlDefn->MsgUserData
                            , MQ_EXIT_DATA_LENGTH
                            , Options
                          );
         } else {
              MQsprintf_X( ParmBuffer
                            , "       MSGDATA(%s) +\n"
                            , pChlDefn->MsgUserData
                            , MQ_TOTAL_EXIT_DATA_LENGTH
                            , Options
                          );
         }
         fputs( ParmBuffer, fp );

         /* [v530] Depending on whether we're dealing with a /400   */
         /* or chained exits, pass the correct flags to MQsprintf_X */
         Options = MQSPRINTF_X_LINE_BREAKS;
         if (MQAT_DEFAULT != MQAT_OS400)
              Options += MQSPRINTF_X_NO_TRAILING_BLANKS;

         if (! pChlDefn->MsgExitListFlag ) {
              Options += MQSPRINTF_X_DOUBLE_QUOTES;
              MQsprintf_X( ParmBuffer
                            , "       MSGEXIT('%s') +\n"
                            , pChlDefn->MsgExit
                            , MQ_EXIT_NAME_LENGTH
                            , Options
                          );
         } else {
              MQsprintf_X( ParmBuffer
                            , "       MSGEXIT(%s) +\n"
                            , pChlDefn->MsgExit
                            , MQ_TOTAL_EXIT_NAME_LENGTH
                            , Options
                          );
         }
         fputs( ParmBuffer, fp );

      }

      #if defined (MQCHT_CLUSRCVR)
      if (UserVersion>=MQCMDL_LEVEL_510 && (
          pChlDefn->ChannelType == MQCHT_CLUSRCVR
          )) {
             sprintf( ParmBuffer , "       NETPRTY(%ld) +\n"
                  , pChlDefn->NetworkPriority );
             fputs( ParmBuffer, fp );
         } /* end of channel = cluster */
      #endif



      if (UserVersion>=MQCMDL_LEVEL_500 && (
            pChlDefn->ChannelType == MQCHT_SENDER
         || pChlDefn->ChannelType == MQCHT_SERVER
         || pChlDefn->ChannelType == MQCHT_RECEIVER
         || pChlDefn->ChannelType == MQCHT_REQUESTER
         #if defined (MQCHT_CLUSSDR)
         || pChlDefn->ChannelType == MQCHT_CLUSSDR
         || pChlDefn->ChannelType == MQCHT_CLUSRCVR
         #endif
         )) {
         #if defined (MQNPMS_NORMAL)
            if ( pChlDefn->NonPersistentMsgSpeed == MQNPMS_NORMAL )
               sprintf( ParmBuffer, "       NPMSPEED(NORMAL) +\n" );
            else
               sprintf( ParmBuffer, "       NPMSPEED(FAST) +\n" );
            fputs( ParmBuffer, fp );
         #endif
      }

      /* Sorry, we don't generate passwords, change the #if you want them */
#if 0
      if (  pChlDefn->ChannelType == MQCHT_SENDER
         || pChlDefn->ChannelType == MQCHT_REQUESTER
         || pChlDefn->ChannelType == MQCHT_CLNTCONN
         || pChlDefn->ChannelType == MQCHT_SERVER
         #if defined (MQCHT_CLUSSDR)
         || pChlDefn->ChannelType == MQCHT_CLUSSDR
         #endif
         ) {
             sprintf( ParmBuffer
                   , "       PASSWORD('%s') +\n"
                   , pChlDefn->Password
                  );
             fputs( ParmBuffer, fp );
         }
#endif

      if (  pChlDefn->ChannelType == MQCHT_RECEIVER
         || pChlDefn->ChannelType == MQCHT_REQUESTER
/* doc says svrconn are okay for putaut ... doesn't make sense to me */
/* and also doesn't work !!                                          */
/*       || pChlDefn->ChannelType == MQCHT_SVRCONN                     */
         #if defined (MQCHT_CLUSRCVR)
         || pChlDefn->ChannelType == MQCHT_CLUSRCVR
         #endif
         ) {

         switch (pChlDefn->PutAuthority) {
             case MQPA_DEFAULT:
                sprintf( ParmBuffer, "       PUTAUT(DEF) +\n" );
               break;
             case  MQPA_CONTEXT:
                sprintf( ParmBuffer, "       PUTAUT(CTX) +\n" );
               break;
             #if defined (MQPA_ONLY_MCA)
             case  MQPA_ONLY_MCA:
                sprintf( ParmBuffer, "       PUTAUT(ONLYMCA) +\n" );
               break;
             #endif
             #if defined (MQPA_ALTERNATE_OR_MCA)
             case  MQPA_ALTERNATE_OR_MCA:
                sprintf( ParmBuffer, "       PUTAUT(ALTMCA) +\n" );
               break;
             #endif
             default:
                sprintf( ParmBuffer, "       PUTAUT(%ld?) +\n"
                         , pChlDefn->PutAuthority );
               break;
           }/* End switch*/
         fputs( ParmBuffer, fp );
         }

      if (  pChlDefn->ChannelType == MQCHT_CLNTCONN ) {

         MQsprintf( ParmBuffer
                  , "       QMNAME('%s') +\n"
                  , pChlDefn->QMgrName
                  , MQ_Q_MGR_NAME_LENGTH
                  );
         fputs( ParmBuffer, fp );

      }

         /* [v530] Depending on whether we're dealing with a string */
         /* or chained exits, pass the correct flags to MQsprintf_X */
         Options = MQSPRINTF_X_LINE_BREAKS + MQSPRINTF_X_NO_TRAILING_BLANKS;

         if (! pChlDefn->ReceiveUserDataListFlag ) {
              Options += MQSPRINTF_X_DOUBLE_QUOTES;
              MQsprintf_X( ParmBuffer
                            , "       RCVDATA('%s') +\n"
                            , pChlDefn->ReceiveUserData
                            , MQ_EXIT_DATA_LENGTH
                            , Options
                          );
         } else {
              MQsprintf_X( ParmBuffer
                            , "       RCVDATA(%s) +\n"
                            , pChlDefn->ReceiveUserData
                            , MQ_TOTAL_EXIT_DATA_LENGTH
                            , Options
                          );
         }
         fputs( ParmBuffer, fp );

      /* [v530] Depending on whether we're dealing with a /400   */
      /* or chained exits, pass the correct flags to MQsprintf_X */
      Options = MQSPRINTF_X_LINE_BREAKS;
      if (MQAT_DEFAULT != MQAT_OS400)
           Options += MQSPRINTF_X_NO_TRAILING_BLANKS;

      if (! pChlDefn->ReceiveExitListFlag ) {
           Options += MQSPRINTF_X_DOUBLE_QUOTES;
           MQsprintf_X( ParmBuffer
                         , "       RCVEXIT('%s') +\n"
                         , pChlDefn->ReceiveExit
                         , MQ_EXIT_NAME_LENGTH
                         , Options
                       );
      } else {
           MQsprintf_X( ParmBuffer
                         , "       RCVEXIT(%s) +\n"
                         , pChlDefn->ReceiveExit
                         , MQ_TOTAL_EXIT_NAME_LENGTH
                         , Options
                       );
      }
      fputs( ParmBuffer, fp );

      MQsprintf( ParmBuffer
               , "       SCYDATA('%s') +\n"
               , pChlDefn->SecurityUserData
               , MQ_EXIT_DATA_LENGTH
               );
      fputs( ParmBuffer, fp );

       /* [v530] Depending on whether we're dealing with a /400   */
       /* or chained exits, pass the correct flags to MQsprintf_X */
       Options = MQSPRINTF_X_LINE_BREAKS + MQSPRINTF_X_DOUBLE_QUOTES;
       if (MQAT_DEFAULT != MQAT_OS400)
            Options += MQSPRINTF_X_NO_TRAILING_BLANKS;

       MQsprintf_X( ParmBuffer
                     , "       SCYEXIT('%s') +\n"
                     , pChlDefn->SecurityExit
                     , MQ_EXIT_NAME_LENGTH
                     , Options
                        );
       fputs( ParmBuffer, fp );


         /* [v530] Depending on whether we're dealing with a string */
         /* or chained exits, pass the correct flags to MQsprintf_X */
         Options = MQSPRINTF_X_LINE_BREAKS + MQSPRINTF_X_NO_TRAILING_BLANKS;

         if (! pChlDefn->SendUserDataListFlag ) {
              Options += MQSPRINTF_X_DOUBLE_QUOTES;
              MQsprintf_X( ParmBuffer
                            , "       SENDDATA('%s') +\n"
                            , pChlDefn->SendUserData
                            , MQ_EXIT_DATA_LENGTH
                            , Options
                          );
         } else {
              MQsprintf_X( ParmBuffer
                            , "       SENDDATA(%s) +\n"
                            , pChlDefn->SendUserData
                            , MQ_TOTAL_EXIT_DATA_LENGTH
                            , Options
                          );
         }
         fputs( ParmBuffer, fp );

        /* [v530] Depending on whether we're dealing with a /400   */
        /* or chained exits, pass the correct flags to MQsprintf_X */
        Options = MQSPRINTF_X_LINE_BREAKS;
        if (MQAT_DEFAULT != MQAT_OS400)
             Options += MQSPRINTF_X_NO_TRAILING_BLANKS;

        if (! pChlDefn->SendExitListFlag ) {
             Options += MQSPRINTF_X_DOUBLE_QUOTES;
             MQsprintf_X( ParmBuffer
                           , "       SENDEXIT('%s') +\n"
                           , pChlDefn->SendExit
                           , MQ_EXIT_NAME_LENGTH
                           , Options
                         );
        } else {
             MQsprintf_X( ParmBuffer
                           , "       SENDEXIT(%s) +\n"
                           , pChlDefn->SendExit
                           , MQ_TOTAL_EXIT_NAME_LENGTH
                           , Options
                         );
        }
        fputs( ParmBuffer, fp );


      if (  pChlDefn->ChannelType == MQCHT_SENDER
         || pChlDefn->ChannelType == MQCHT_SERVER
         || pChlDefn->ChannelType == MQCHT_RECEIVER
         || pChlDefn->ChannelType == MQCHT_REQUESTER
         #if defined (MQCHT_CLUSSDR)
         || pChlDefn->ChannelType == MQCHT_CLUSSDR
         || pChlDefn->ChannelType == MQCHT_CLUSRCVR
         #endif
         ) {

         sprintf( ParmBuffer, "       SEQWRAP(%ld) +\n",
                  pChlDefn->SeqNumberWrap );
         fputs( ParmBuffer, fp );

         }


      if (MQAT_DEFAULT != MQAT_OS400 &&
          UserVersion > MQCMDL_LEVEL_110 && (
            pChlDefn->ChannelType == MQCHT_SENDER
         || pChlDefn->ChannelType == MQCHT_REQUESTER
         || pChlDefn->ChannelType == MQCHT_CLNTCONN
         || pChlDefn->ChannelType == MQCHT_SERVER
         #if defined (MQCHT_CLUSSDR)
         || pChlDefn->ChannelType == MQCHT_CLUSSDR
         #endif
         )) {

            MQsprintf( ParmBuffer
                     , "       USERID('%s') +\n"
                     , pChlDefn->UserIdentifier
                     , MQ_USER_ID_LENGTH
                     );
            fputs( ParmBuffer, fp );
         }

      if (  pChlDefn->ChannelType == MQCHT_SENDER
         || pChlDefn->ChannelType == MQCHT_SERVER
         ) {

         MQsprintf( ParmBuffer
                  , "       XMITQ('%s') +\n"
                  , pChlDefn->XmitQName
                  , MQ_Q_NAME_LENGTH
                  );
         fputs( ParmBuffer, fp );
         }


      /* [V520] Include Tandem NSK channel sttribute */
   #ifdef MQIACH_INHIBIT_AUTOSTART
      if (  pChlDefn->ChannelType == MQCHT_SERVER
         || pChlDefn->ChannelType == MQCHT_REQUESTER
         || pChlDefn->ChannelType == MQCHT_SVRCONN
         || pChlDefn->ChannelType == MQCHT_RECEIVER
            ) {
           if ( pChlDefn->InhibitAutostart == MQCFC_AUTOSTART_ALLOWED) {
              sprintf( ParmBuffer, "       AUTOSTART(ENABLED) +\n" );
              fputs( ParmBuffer, fp );
           } else {
              sprintf( ParmBuffer, "       AUTOSTART(DISABLED) +\n" );
              fputs( ParmBuffer, fp );
           } /* endif */
      }
    #endif


      /* [V530] Include SSL Client Auth Attribute for receiving type chls */
   #ifdef MQIACH_SSL_CLIENT_AUTH
      if (  pChlDefn->ChannelType == MQCHT_SERVER
         || pChlDefn->ChannelType == MQCHT_REQUESTER
         || pChlDefn->ChannelType == MQCHT_SVRCONN
         || pChlDefn->ChannelType == MQCHT_CLUSRCVR
         || pChlDefn->ChannelType == MQCHT_RECEIVER
            ) {
          if (UserVersion >= MQCMDL_LEVEL_530) {
                if ( pChlDefn->SslCAuth ) {
                     sprintf( ParmBuffer, "       SSLCAUTH(OPTIONAL) +\n" );
                } else {
                     sprintf( ParmBuffer, "       SSLCAUTH(REQUIRED) +\n" );
               } /* endif */
                fputs( ParmBuffer, fp );
           }
      }
    #endif

     /* [v530] all channel types get SSLCIPH and SSLPEER */
        #if defined (MQCACH_SSL_PEER_NAME)

            if (UserVersion>=MQCMDL_LEVEL_530) {

                MQsprintf( ParmBuffer
                         , "       SSLCIPH('%s') +\n"
                         , pChlDefn->SslCipher
                         , MQ_SSL_CIPHER_SPEC_LENGTH );
                fputs( ParmBuffer, fp );

                MQsprintf( ParmBuffer
                         , "       SSLPEER('%s') +\n"
                         , pChlDefn->SslPeer
                         , MQ_SSL_PEER_NAME_LENGTH );
                fputs( ParmBuffer, fp );

            }
        #endif
 
        /* [v531] csd01 on v5.3 adds KAINT to mqsc */
        #if defined (MQIACH_KEEP_ALIVE_INTERVAL)
            if (UserVersion>=MQCMDL_LEVEL_530) {
	      if (pChlDefn->KAInt != -999) {  /* -999 means we never got it from mqsc */
               if (pChlDefn->KAInt == -1) {
                    sprintf( ParmBuffer
                         , "        KAINT(AUTO) +\n"
                         );
                 } else {
                    sprintf( ParmBuffer
                         , "        KAINT(%ld) +\n"
                         , pChlDefn->KAInt );
                 }
                 fputs( ParmBuffer, fp );
                 pChlDefn -> KAInt = -999;   /* special value to indicate not to set KAINT */  
	     }
	    }
     #endif


      sprintf( ParmBuffer, "       REPLACE\n\n" );
      fputs( ParmBuffer, fp );


      /* v512 -- if -R specified, then create "reset" commands */
      if ( chlReset && (
            pChlDefn->ChannelType == MQCHT_SENDER
         || pChlDefn->ChannelType == MQCHT_SERVER
         || pChlDefn->ChannelType == MQCHT_RECEIVER
         || pChlDefn->ChannelType == MQCHT_REQUESTER
         #if defined (MQCHT_CLUSSDR)
         || pChlDefn->ChannelType == MQCHT_CLUSSDR
         || pChlDefn->ChannelType == MQCHT_CLUSRCVR
         #endif
         )) {
              MQsprintf( ParmBuffer
                , "RESET CHANNEL ('%s') SEQNUM(1)\n\n"
                , pChlDefn->ChannelName
                , MQ_CHANNEL_NAME_LENGTH
                );
              fputs( ParmBuffer, fp );

         }

      free (ParmBuffer);
   return;
}


