/*===========================================================================*/
/*              This supportpac is maintained by Wayne M. Schutz             */
/*                                                                           */
/*              Chris Petty is the original author.                          */
/*                                                                           */
/*  (C) Copyright IBM Corporation 1995, 1997, 1998, 1999, 2000, 2002, 2003   */
/*===========================================================================*/
/*                                                                           */
/*  See the read.me file for change history                                  */
/*                                                                           */
/*===========================================================================*/
/* Module Name: PROCESS.C                                                    */
/*===========================================================================*/

/* Include standard libraries */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

/* Include MQSeries headers */
#include <cmqc.h>
#include <cmqcfc.h>
#include <cmqxc.h>


/* Include Application headers */
#include "saveqmgr.h"



/* ------------------------------------------------------------------------ */
/* ProcessProcStringParm()                                                  */
/* ProcessProcIntegerParm()                                                 */
/*                                                                          */
/* These two processes are given a pointer to a block of storage that       */
/* contains the definition of a single attribute relating to an individual  */
/* process. The first handles string parameters, the second integer.        */
/* The calling process has already decided which type of parameter this is  */
/* and is responsible for managing the storage. These procedures decide     */
/* which of the possible attributes this one is and extract the value.      */
/*                                                                          */
/* Follows is a list of all attributes possible for a process definition.   */
/*                                                                          */
/* The ProcDefn structure contains entries for each of the attributes.      */
/* ------------------------------------------------------------------------ */
extern int noisy;

void ProcessProcStringParm( MQCFST    *pPCFString
                          , ProcParms *ProcDefn
                          , MQLONG       *iCC
                          )
{
   /* Initialise return codes */
   *iCC = OK;

   switch ( pPCFString->Parameter ) {
   case MQCA_PROCESS_NAME:
      memset( ProcDefn->ProcessName
            , ' ', MQ_PROCESS_NAME_LENGTH );
      memcpy( ProcDefn->ProcessName
            , pPCFString->String, pPCFString->StringLength );
      break;
   case MQCA_PROCESS_DESC:
      memset( ProcDefn->ProcessDesc
            , ' ', MQ_PROCESS_DESC_LENGTH );
      memcpy( ProcDefn->ProcessDesc
            , pPCFString->String, pPCFString->StringLength );
      break;
   case MQCA_APPL_ID:
      memset( ProcDefn->ApplID
            , ' ', MQ_PROCESS_APPL_ID_LENGTH );
      memcpy( ProcDefn->ApplID
            , pPCFString->String, pPCFString->StringLength );
      break;
   case MQCA_ENV_DATA:
      memset( ProcDefn->EnvData
            , ' ', MQ_PROCESS_ENV_DATA_LENGTH );
      memcpy( ProcDefn->EnvData
            , pPCFString->String, pPCFString->StringLength );
      break;
   case MQCA_USER_DATA:
      memset( ProcDefn->UserData
            , ' ', MQ_PROCESS_USER_DATA_LENGTH );
      memcpy( ProcDefn->UserData
            , pPCFString->String, pPCFString->StringLength );
      break;

/* [v51] begin v51 attributes */
#if defined (MQCA_ALTERATION_DATE)
   case MQCA_ALTERATION_DATE:
      memset( ProcDefn->AlterationDate
            , ' ', MQ_CREATION_DATE_LENGTH );
      memcpy( ProcDefn->AlterationDate
            , pPCFString->String, pPCFString->StringLength);
      break;
   case MQCA_ALTERATION_TIME:
      memset( ProcDefn->AlterationTime
            , ' ', MQ_CREATION_TIME_LENGTH );
      memcpy( ProcDefn->AlterationTime
            , pPCFString->String, pPCFString->StringLength);
      break;
#endif
/* [v51] end v51 attributes */
   default:
       if(noisy) fprintf(stderr, "Unknown PCF String Parameter: %ld\n",
              pPCFString->Parameter);
      *iCC = WARNING;
      break;
   } /* endswitch */

   return;
}


void ProcessProcIntegerParm( MQCFIN    *pPCFInteger
                           , ProcParms *ProcDefn
                           , MQLONG       *iCC
                           )
{
   /* Initialise return codes */
   *iCC = OK;

   switch ( pPCFInteger->Parameter ) {
   case MQIA_APPL_TYPE:
      ProcDefn->ApplType = pPCFInteger->Value;
      break;
   default:
       if(noisy) fprintf(stderr, "Unknown PCF Integer Parameter: %ld\n",
              pPCFInteger->Parameter);
      *iCC = WARNING;
      break;
   } /* endswitch */

   return;
}


/* ------------------------------------------------------------------------ */
/*                                                                          */
/* This process takes the attributes of a single process and adds them      */
/* to the end of a file, SAVEQMGR.TST, which can be found in the current    */
/* directory.                                                               */
/*                                                                          */
/* The file is of a format suitable for subsequent input to RUNMQSC.        */
/*                                                                          */
/* ------------------------------------------------------------------------ */
void AddToFilePROCESS(
                       ProcParms *pProcDefn,
                       int UserVersion,
                       char * fileName,   /* v32 Name of the output file  */
                        FILE * fp,       /* v51 */
                       MQLONG *iCC )
{
   char    * ParmBuffer;   /* Temporary buffer to hold for output to file */

   if ( !( ParmBuffer = (char *) malloc (PBSIZE) )) {
     fprintf (stderr, "Failed to allocate %d bytes \n", PBSIZE);
     exit(2);
   }

   /* Initialise return codes */
   *iCC = OK;

   /* Append these details to the end of the current output file (v32) */
   /* v51 removes open from here ... fp = fopen( fileName, OPENMODE ); */

   MQsprintf( ParmBuffer
            , "DEFINE PROCESS ('%s') REPLACE +\n"
            , pProcDefn->ProcessName
            , MQ_PROCESS_NAME_LENGTH
            );
   fputs( ParmBuffer, fp );

   #if defined (MQCA_ALTERATION_DATE)
   MQsprintf( ParmBuffer, "* ALTDATE (%s) + \n",
            pProcDefn->AlterationDate,
            MQ_CREATION_DATE_LENGTH);
   fputs( ParmBuffer, fp);
   MQsprintf( ParmBuffer, "* ALTTIME (%s) + \n",
            pProcDefn->AlterationTime,
            MQ_CREATION_TIME_LENGTH);
   fputs( ParmBuffer, fp);
   #endif

   MQsprintf( ParmBuffer
            , "       DESCR('%s') +\n"
            , pProcDefn->ProcessDesc
            , MQ_PROCESS_DESC_LENGTH
            );
   fputs( ParmBuffer, fp );

   switch ( pProcDefn->ApplType ) {
   case MQAT_OS400:
      sprintf( ParmBuffer, "       APPLTYPE(OS400) +\n" );
      fputs( ParmBuffer, fp );
      break;
   case MQAT_OS2:
      sprintf( ParmBuffer, "       APPLTYPE(OS2) +\n" );
      fputs( ParmBuffer, fp );
      break;
   case MQAT_DOS:
      sprintf( ParmBuffer, "       APPLTYPE(DOS) +\n" );
      fputs( ParmBuffer, fp );
      break;
   case MQAT_WINDOWS:
      sprintf( ParmBuffer, "       APPLTYPE(WINDOWS) +\n" );
      fputs( ParmBuffer, fp );
      break;
      /* [WMS] AIX is now UNIX */
 /*      case MQAT_AIX:                                        */
 /*       sprintf( ParmBuffer, "       APPLTYPE(AIX) +\n" ); */
 /*       fputs( ParmBuffer, fp );                           */
 /*       break;                                             */

   case MQAT_CICS:
      sprintf( ParmBuffer, "       APPLTYPE(CICS) +\n" );
      fputs( ParmBuffer, fp );
      break;
   /* [WMS] adding "new" application types */
   case MQAT_QMGR:
      sprintf( ParmBuffer, "       APPLTYPE(QMGR) +\n" );
      fputs( ParmBuffer, fp );
      break;
   case MQAT_IMS:
      sprintf( ParmBuffer, "       APPLTYPE(IMS) +\n" );
      fputs( ParmBuffer, fp );
      break;
   case MQAT_CICS_VSE:
      sprintf( ParmBuffer, "       APPLTYPE(CICSVSE) +\n" );
      fputs( ParmBuffer, fp );
      break;
   case MQAT_UNIX:
      sprintf( ParmBuffer, "       APPLTYPE(UNIX) +\n" );
      fputs( ParmBuffer, fp );
      break;

/*   [OvE] On VMS the APPLTYPE parameter is not accepted:         */
/*         APPLTYPE(OpenVMS) is filled in by default when omitted */
/* v513 - [wms] new version now accepts "OpenVMS" as appltype     */

     #if defined (MQAT_VMS)
     case MQAT_VMS:
        sprintf( ParmBuffer, "       APPLTYPE(OpenVMS) +\n" );
        fputs( ParmBuffer, fp );
        break;
     #endif
   #if defined (MQAT_VOS)
   case MQAT_VOS:
      sprintf( ParmBuffer, "       APPLTYPE(VOS) +\n" );
      fputs( ParmBuffer, fp );
      break;
   #endif
   #if defined (MQAT_GUARDIAN)
   case MQAT_GUARDIAN:
      sprintf( ParmBuffer, "       APPLTYPE(GUARDIAN) +\n" );
      fputs( ParmBuffer, fp );
      break;
   #endif
   #if defined (MQAT_IMS_BRIDGE)
   case MQAT_IMS_BRIDGE:
      sprintf( ParmBuffer, "       APPLTYPE(IMS_BRIDGE) +\n" );
      fputs( ParmBuffer, fp );
      break;
   #endif
   #if defined (MQAT_WINDOWS_NT)
   case MQAT_WINDOWS_NT:
      sprintf( ParmBuffer, "       APPLTYPE(WINDOWSNT) +\n" );
      fputs( ParmBuffer, fp );
      break;
   #endif
   #if defined (MQAT_XCF)
   case MQAT_XCF:
      sprintf( ParmBuffer, "       APPLTYPE(XCF) +\n" );
      fputs( ParmBuffer, fp );
      break;
   #endif
   default :
      /* User defined application type */
      sprintf( ParmBuffer, "       APPLTYPE(%ld) +\n", pProcDefn->ApplType );
      fputs( ParmBuffer, fp );
      break;
   } /* endswitch */

   MQsprintf( ParmBuffer
            , "       APPLICID('%s') +\n"
            , pProcDefn->ApplID
            , MQ_PROCESS_APPL_ID_LENGTH
            );
   fputs( ParmBuffer, fp );

   MQsprintf( ParmBuffer
            , "       USERDATA('%s') +\n"
            , pProcDefn->UserData
            , MQ_PROCESS_USER_DATA_LENGTH
            );
   fputs( ParmBuffer, fp );

   MQsprintf( ParmBuffer
            , "       ENVRDATA('%s')\n"
            , pProcDefn->EnvData
            , MQ_PROCESS_ENV_DATA_LENGTH
            );
   fputs( ParmBuffer, fp );

   sprintf( ParmBuffer, "\n" );
   fputs( ParmBuffer, fp );

   /* v51 removes close from here ... fclose(fp); */

   free (ParmBuffer);
   return;

}


