/*===========================================================================*/
/*              This supportpac is maintained by Wayne M. Schutz             */
/*                                                                           */
/*              Chris Petty is the original author.                          */
/*                                                                           */
/*  (C) Copyright IBM Corporation 1995, 1997, 1998, 1999, 2000, 2002, 2003   */
/*===========================================================================*/
/*                                                                           */
/*  See the read.me file for change history                                  */
/*                                                                           */
/*===========================================================================*/
/* Module Name: QMGR.C                                                       */
/*===========================================================================*/

/* Include standard libraries */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

/* Include MQSeries headers */
#include <cmqc.h>
#include <cmqcfc.h>
#include <cmqxc.h>

/* move these includes up for Solaris */
#include <sys/types.h>
#include  <time.h>
/* INCLUDE FOR TIME FUNCTION */
#if (MQAT_DEFAULT != MQAT_OS400)
#include <sys/timeb.h>
#endif

/* Include Application headers */
#include "saveqmgr.h"

/* ------------------------------------------------------------------------ */
/* ProcessQMgrStringParm()                                                  */
/* ProcessQMgrIntegerParm()                                                 */
/*                                                                          */
/* These two processes are given a pointer to a block of storage that       */
/* contains the definition of a single attribute relating to the            */
/* queue manager. The first handles string parameters, the second integer.  */
/* The calling process has already decided which type of parameter this is  */
/* and is responsible for managing the storage. These procedures decide     */
/* which of the possible attributes this one is and extract the value.      */
/*                                                                          */
/* Follows is a list of all attributes possible for a queue manager.        */
/*                                                                          */
/* The QMgrDefn structure contains entries for each of the attributes.      */
/* ------------------------------------------------------------------------ */
extern int noisy;
extern int header; /* v513 */

void ProcessQMgrStringParm( MQCFST    *pPCFString
                          , QMgrParms *QMgrDefn
                          , MQLONG       *iCC
                          )
{
   /* Initialise return codes */
   *iCC = OK;

   switch ( pPCFString->Parameter ) {
   case MQCA_Q_MGR_NAME:
      memset( QMgrDefn->QMgrName
            , ' ', MQ_Q_MGR_NAME_LENGTH );
      memcpy( QMgrDefn->QMgrName
            , pPCFString->String, pPCFString->StringLength);
      break;
   case MQCA_Q_MGR_DESC:
      memset( QMgrDefn->QMgrDesc
            , ' ', MQ_Q_MGR_DESC_LENGTH );
      memcpy( QMgrDefn->QMgrDesc
            , pPCFString->String, pPCFString->StringLength);
      break;
   case MQCA_DEAD_LETTER_Q_NAME:
      memset( QMgrDefn->DeadLetterQName
            , ' ', MQ_Q_NAME_LENGTH );
      memcpy( QMgrDefn->DeadLetterQName
            , pPCFString->String, pPCFString->StringLength);
      break;
   case MQCA_COMMAND_INPUT_Q_NAME:
      memset( QMgrDefn->CommandInputQName
            , ' ', MQ_Q_NAME_LENGTH );
      memcpy( QMgrDefn->CommandInputQName
            , pPCFString->String, pPCFString->StringLength);
      break;
   case MQCA_DEF_XMIT_Q_NAME:
      memset( QMgrDefn->DefXmitQName
            , ' ', MQ_Q_NAME_LENGTH  );
      memcpy( QMgrDefn->DefXmitQName
            , pPCFString->String, pPCFString->StringLength);
      break;
#if defined (MQCA_CHANNEL_AUTO_DEF_EXIT)
   /* [wms]  start V5 attributes */
   case MQCA_CHANNEL_AUTO_DEF_EXIT:
      memset( QMgrDefn->ChannelAutoDefExit
            , ' ', MQ_EXIT_NAME_LENGTH  );
      memcpy( QMgrDefn->ChannelAutoDefExit
            , pPCFString->String, pPCFString->StringLength);
      break;
   /* [wms]   end V5 attributes */
#endif

   /* [wms]  start V51 attributes */
#if defined (MQCA_CLUSTER_WORKLOAD_EXIT)
   case MQCA_CLUSTER_WORKLOAD_EXIT:
      memset( QMgrDefn->WorkLoadExit
            , ' ', MQ_EXIT_NAME_LENGTH  );
      memcpy( QMgrDefn->WorkLoadExit
            , pPCFString->String, pPCFString->StringLength);
      break;
#endif
#if defined (MQCA_CLUSTER_WORKLOAD_DATA)
   case MQCA_CLUSTER_WORKLOAD_DATA:
      memset( QMgrDefn->WorkLoadData
            , ' ', MQ_EXIT_DATA_LENGTH  );
      memcpy( QMgrDefn->WorkLoadData
            , pPCFString->String, pPCFString->StringLength);
      break;
#endif
#if defined (MQCA_REPOSITORY_NAME)
   case MQCA_REPOSITORY_NAME:
      memset( QMgrDefn->RepositoryName
            , ' ', MQ_OBJECT_NAME_LENGTH  );
      memcpy( QMgrDefn->RepositoryName
            , pPCFString->String, pPCFString->StringLength);
      break;
#endif
#if defined (MQCA_REPOSITORY_NAMELIST)
   case MQCA_REPOSITORY_NAMELIST:
      memset( QMgrDefn->RepositoryNameList
            , ' ', MQ_OBJECT_NAME_LENGTH  );
      memcpy( QMgrDefn->RepositoryNameList
            , pPCFString->String, pPCFString->StringLength);
      break;
#endif
#if defined (MQCA_ALTERATION_DATE)
   case MQCA_CREATION_DATE:
      memset( QMgrDefn->CreationDate
            , ' ', MQ_CREATION_DATE_LENGTH );
      memcpy( QMgrDefn->CreationDate
            , pPCFString->String, pPCFString->StringLength);
      break;
   case MQCA_CREATION_TIME:
      memset( QMgrDefn->CreationTime
            , ' ', MQ_CREATION_TIME_LENGTH );
      memcpy( QMgrDefn->CreationTime
            , pPCFString->String, pPCFString->StringLength);
      break;
   case MQCA_ALTERATION_DATE:
      memset( QMgrDefn->AlterationDate
            , ' ', MQ_CREATION_DATE_LENGTH );
      memcpy( QMgrDefn->AlterationDate
            , pPCFString->String, pPCFString->StringLength);
      break;
   case MQCA_ALTERATION_TIME:
      memset( QMgrDefn->AlterationTime
            , ' ', MQ_CREATION_TIME_LENGTH );
      memcpy( QMgrDefn->AlterationTime
            , pPCFString->String, pPCFString->StringLength);
      break;
#endif
#if defined (MQCA_Q_MGR_IDENTIFIER)
   case MQCA_Q_MGR_IDENTIFIER:
      memset( QMgrDefn->QMgrId
            , ' ', MQ_Q_MGR_IDENTIFIER_LENGTH );
      memcpy( QMgrDefn->QMgrId
            , pPCFString->String, pPCFString->StringLength);
      break;
#endif

      /* [v530] New SSL Attributes: */
#if defined (MQCA_SSL_CRL_NAMELIST)
   case MQCA_SSL_CRL_NAMELIST:
      memset( QMgrDefn->SslCrlNameList
            , ' ', MQ_OBJECT_NAME_LENGTH );
      memcpy( QMgrDefn->SslCrlNameList
            , pPCFString->String, pPCFString->StringLength);
      break;
   case MQCA_SSL_KEY_REPOSITORY:
      memset( QMgrDefn->SslKeyRepos
            , ' ', MQ_SSL_KEY_REPOSITORY_LENGTH );
      memcpy( QMgrDefn->SslKeyRepos
            , pPCFString->String, pPCFString->StringLength);
      break;
   case MQCA_SSL_CRYPTO_HARDWARE:
      memset( QMgrDefn->SslCryptoHW
            , ' ', MQ_SSL_CRYPTO_HARDWARE_LENGTH );
      memcpy( QMgrDefn->SslCryptoHW
            , pPCFString->String, pPCFString->StringLength);
      break;

#endif

/* [v51] end of V51 attributes */
   default:
       if(noisy) fprintf(stderr, "Unknown PCF String Parameter: %ld\n",
              pPCFString->Parameter);
      *iCC = WARNING;
      break;
   } /* endswitch */

   return;
}


void ProcessQMgrIntegerParm( MQCFIN    *pPCFInteger
                          , QMgrParms *QMgrDefn
                          , MQLONG       *iCC
                          )
{
   /* Initialise Completion and Reason Codes */
   *iCC = OK;

   switch ( pPCFInteger->Parameter ) {
   case MQIA_PLATFORM:
      QMgrDefn->Platform = pPCFInteger->Value;
      break;
   case MQIA_COMMAND_LEVEL:
      QMgrDefn->CommandLevel = pPCFInteger->Value;
      break;
   case MQIA_TRIGGER_INTERVAL:
      QMgrDefn->TriggerInterval = pPCFInteger->Value;
      break;
   case MQIA_MAX_PRIORITY:
      QMgrDefn->MaxPriority = pPCFInteger->Value;
      break;
   case MQIA_CODED_CHAR_SET_ID:
      QMgrDefn->CodedCharSetId = pPCFInteger->Value;
      break;
   case MQIA_MAX_HANDLES:
      QMgrDefn->MaxHandles = pPCFInteger->Value;
      break;
   case MQIA_MAX_MSG_LENGTH:
      QMgrDefn->MaxMsgLength = pPCFInteger->Value;
      break;
   case MQIA_MAX_UNCOMMITTED_MSGS:
      QMgrDefn->MaxUncommittedMsgs = pPCFInteger->Value;
      break;
   case MQIA_SYNCPOINT:
      QMgrDefn->SyncPoint = pPCFInteger->Value;
      break;
   case MQIA_AUTHORITY_EVENT:
      QMgrDefn->AuthorityEvent = pPCFInteger->Value;
      break;
   case MQIA_INHIBIT_EVENT:
      QMgrDefn->InhibitEvent = pPCFInteger->Value;
      break;
   case MQIA_LOCAL_EVENT:
      QMgrDefn->LocalEvent = pPCFInteger->Value;
      break;
   case MQIA_REMOTE_EVENT:
      QMgrDefn->RemoteEvent = pPCFInteger->Value;
      break;
   case MQIA_START_STOP_EVENT:
      QMgrDefn->StartStopEvent = pPCFInteger->Value;
      break;
   case MQIA_PERFORMANCE_EVENT:
      QMgrDefn->PerformanceEvent = pPCFInteger->Value;
      break;
#if defined (MQIA_CHANNEL_AUTO_DEF)
   /* [wms] begin V5 attributes */
   case MQIA_CHANNEL_AUTO_DEF:
      QMgrDefn->ChannelAutoDef  = pPCFInteger->Value;
      break;
#endif
#if defined (MQIA_CHANNEL_AUTO_DEF_EVENT)
   case MQIA_CHANNEL_AUTO_DEF_EVENT:
      QMgrDefn->ChannelAutoDefEvent = pPCFInteger->Value;
      break;
#endif
#if defined (MQIA_DIST_LISTS)
   case MQIA_DIST_LISTS:
      QMgrDefn->DistLists = pPCFInteger->Value;
      break;
   /* [wms]   end V5 attributes */

   /* [v51] begin v51 attributes */
#if defined (MQIA_CLUSTER_WORKLOAD_LENGTH)
   case MQIA_CLUSTER_WORKLOAD_LENGTH:
      QMgrDefn->WorkLoadLength = pPCFInteger->Value;
      break;
   /* [v51]   end V51 attributes */
#endif
#endif
   default:
       if(noisy) fprintf(stderr, "Unknown PCF Integer Parameter: %ld\n",
              pPCFInteger->Parameter);
      *iCC = WARNING;
      break;
   } /* endswitch */

}


/* ------------------------------------------------------------------------ */
/*                                                                          */
/* This process takes the attributes of the local queue manager and adds    */
/* them to the end of a file, SAVEQMGR.TST, which can be found in the       */
/* current directory.                                                       */
/*                                                                          */
/* The file is of a format suitable for subsequent input to RUNMQSC.        */
/*                                                                          */
/* ------------------------------------------------------------------------ */
void AddToFileQMGR(
                    QMgrParms *pQMgrDefn,
                    int UserVersion,
                    char * fileName,   /* v32 Name of the output file  */
                        FILE * fp,     /* v51 */
                    MQLONG *iCC )
{
   char    * ParmBuffer;   /* Temporary buffer to hold for output to file */
   time_t ltime;
   struct tm *today;

      /* Initialise Completion and Reason Codes */
      *iCC = OK;

   if ( !( ParmBuffer = (char *) malloc (PBSIZE) )) {
     fprintf (stderr, "Failed to allocate %d bytes \n", PBSIZE);
     exit(2);
   }

   /* v5.3 ... clean up when we put out "generating vx.x messages */
   {
     int ver = UserVersion / 100;
     int rel = (UserVersion - (ver * 100)) / 10;
     int mod = UserVersion - (ver * 100) - (rel * 10);
     fprintf(stderr, "Generating attributes for Websphere MQ Release %d.%d.%d\n",
                      ver, rel, mod);
     }



   if (header ) {  /* v513 */

             time( &ltime );
             today = localtime( &ltime );

             fprintf( fp,
               "*\n* This file generated by %s on %.4d-%.2d-%.2d at %.2d.%.2d.%.2d hours.\n",
                   THISVERSION,
                   (today->tm_year) + 1900,
                   today -> tm_mon+1,
                   today -> tm_mday,
                   today -> tm_hour,
                   today -> tm_min,
                   today -> tm_sec);
      }

      MQsprintf( ParmBuffer, "*\n* QMNAME (%s) + \n",
                 pQMgrDefn->QMgrName,
                 MQ_Q_MGR_NAME_LENGTH);
      fputs( ParmBuffer, fp);

      #if defined(MQ_Q_MGR_IDENTIFIER_LENGTH)
      MQsprintf( ParmBuffer, "* CRDATE (%s) + \n",
               pQMgrDefn->CreationDate,
               MQ_CREATION_DATE_LENGTH);
      fputs( ParmBuffer, fp);
      MQsprintf( ParmBuffer, "* CRTIME (%s) + \n",
               pQMgrDefn->CreationTime,
               MQ_CREATION_TIME_LENGTH);
      fputs( ParmBuffer, fp);
      MQsprintf( ParmBuffer, "* ALTDATE (%s) + \n",
               pQMgrDefn->AlterationDate,
               MQ_CREATION_DATE_LENGTH);
      fputs( ParmBuffer, fp);
      MQsprintf( ParmBuffer, "* ALTTIME (%s) + \n",
               pQMgrDefn->AlterationTime,
               MQ_CREATION_TIME_LENGTH);
      fputs( ParmBuffer, fp);
      MQsprintf( ParmBuffer, "* QMID (%s) + \n*\n",
               pQMgrDefn->QMgrId,
               MQ_Q_MGR_IDENTIFIER_LENGTH);
      fputs( ParmBuffer, fp);
      #endif

      sprintf( ParmBuffer, "ALTER  QMGR +\n" );
      fputs( ParmBuffer, fp );

      /* v32 mq/win doesnt support authorev and deadq */
      if (UserVersion > MQCMDL_LEVEL_110){

         if ( pQMgrDefn->AuthorityEvent == MQEVR_DISABLED ) {
            sprintf( ParmBuffer, "       AUTHOREV(DISABLED) +\n" );
            fputs( ParmBuffer, fp );
         } else {
            sprintf( ParmBuffer, "       AUTHOREV(ENABLED) +\n" );
            fputs( ParmBuffer, fp );
         } /* endif */

         MQsprintf( ParmBuffer
                  , "       DEADQ('%s') +\n"
                  , pQMgrDefn->DeadLetterQName
                  , MQ_Q_NAME_LENGTH
                  );
         fputs( ParmBuffer, fp );

        } /* v32 -- dont generate authorev / deadq attribute for mq/win */

      MQsprintf( ParmBuffer
               , "       DEFXMITQ('%s') +\n"
               , pQMgrDefn->DefXmitQName
               , MQ_Q_NAME_LENGTH
               );
      fputs( ParmBuffer, fp );

      MQsprintf( ParmBuffer
               , "       DESCR('%s') +\n"
               , pQMgrDefn->QMgrDesc
               , MQ_Q_MGR_DESC_LENGTH
               );
      fputs( ParmBuffer, fp );

      if ( pQMgrDefn->InhibitEvent == MQEVR_DISABLED ) {
         sprintf( ParmBuffer, "       INHIBTEV(DISABLED) +\n" );
         fputs( ParmBuffer, fp );
      } else {
         sprintf( ParmBuffer, "       INHIBTEV(ENABLED) +\n" );
         fputs( ParmBuffer, fp );
      } /* endif */

      if ( pQMgrDefn->LocalEvent == MQEVR_DISABLED ) {
         sprintf( ParmBuffer, "       LOCALEV(DISABLED) +\n" );
         fputs( ParmBuffer, fp );
      } else {
         sprintf( ParmBuffer, "       LOCALEV(ENABLED) +\n" );
         fputs( ParmBuffer, fp );
      } /* endif */

      sprintf( ParmBuffer, "       MAXHANDS(%ld) +\n", pQMgrDefn->MaxHandles );
      fputs( ParmBuffer, fp );

      sprintf( ParmBuffer, "       MAXUMSGS(%ld) +\n"
             , pQMgrDefn->MaxUncommittedMsgs );
      fputs( ParmBuffer, fp );

      if ( pQMgrDefn->PerformanceEvent == MQEVR_DISABLED ) {
         sprintf( ParmBuffer, "       PERFMEV(DISABLED) +\n" );
         fputs( ParmBuffer, fp );
      } else {
         sprintf( ParmBuffer, "       PERFMEV(ENABLED) +\n" );
         fputs( ParmBuffer, fp );
      } /* endif */

      if ( pQMgrDefn->RemoteEvent == MQEVR_DISABLED ) {
         sprintf( ParmBuffer, "       REMOTEEV(DISABLED) +\n" );
         fputs( ParmBuffer, fp );
      } else {
         sprintf( ParmBuffer, "       REMOTEEV(ENABLED) +\n" );
         fputs( ParmBuffer, fp );
      } /* endif */

      if ( pQMgrDefn->StartStopEvent == MQEVR_DISABLED ) {
         sprintf( ParmBuffer, "       STRSTPEV(DISABLED) +\n" );
         fputs( ParmBuffer, fp );
      } else {
         sprintf( ParmBuffer, "       STRSTPEV(ENABLED) +\n" );
         fputs( ParmBuffer, fp );
      } /* endif */

      sprintf( ParmBuffer, "       TRIGINT(%ld) +\n", pQMgrDefn->TriggerInterval );
      fputs( ParmBuffer, fp );

      /* [wms] start V5 attributes */
      #if defined (MQCHAD_DISABLED)
      if (UserVersion>=MQCMDL_LEVEL_500)
         {
         /* [wms] v520 ... oops, forgot maxmsgl ...  */
         /* [wms] v530 ... oops, forgot maxmsgl only applies to V5 ! ...  */
         sprintf( ParmBuffer, "       MAXMSGL(%ld) +\n", pQMgrDefn->MaxMsgLength );
         fputs( ParmBuffer, fp );
         if ( pQMgrDefn->ChannelAutoDef == MQCHAD_DISABLED ) {
            sprintf( ParmBuffer, "       CHAD(DISABLED) +\n" );
            fputs( ParmBuffer, fp );
         } else {
            sprintf( ParmBuffer, "       CHAD(ENABLED) +\n" );
            fputs( ParmBuffer, fp );
         } /* endif */
         if ( pQMgrDefn->ChannelAutoDefEvent == MQEVR_DISABLED ) {
            sprintf( ParmBuffer, "       CHADEV(DISABLED) +\n" );
            fputs( ParmBuffer, fp );
         } else {
            sprintf( ParmBuffer, "       CHADEV(ENABLED) +\n" );
            fputs( ParmBuffer, fp );
         } /* endif */
         MQsprintf( ParmBuffer
                  , "       CHADEXIT('%s') +\n"
                  , pQMgrDefn->ChannelAutoDefExit
                  , MQ_EXIT_NAME_LENGTH
                  );
         fputs( ParmBuffer, fp );
      }
      #endif
      /* [wms]   end V5 attributes */

      #if defined (MQ_OBJECT_NAME_LENGTH)
      /* [v51] start V51 attributes */
      if (UserVersion>=MQCMDL_LEVEL_510)
         {
            MQsprintf( ParmBuffer
                     , "       CLWLEXIT('%s') +\n"
                     , pQMgrDefn->WorkLoadExit
                     , MQ_EXIT_NAME_LENGTH
                     );
            fputs( ParmBuffer, fp );
            MQsprintf( ParmBuffer
                     , "       CLWLDATA('%s') +\n"
                     , pQMgrDefn->WorkLoadData
                     , MQ_EXIT_DATA_LENGTH
                     );
            fputs( ParmBuffer, fp );
            MQsprintf( ParmBuffer
                     , "       REPOS('%s') +\n"
                     , pQMgrDefn->RepositoryName
                     , MQ_OBJECT_NAME_LENGTH
                     );
            fputs( ParmBuffer, fp );
            MQsprintf( ParmBuffer
                     , "       REPOSNL('%s') +\n"
                     , pQMgrDefn->RepositoryNameList
                     , MQ_OBJECT_NAME_LENGTH
                     );
            fputs( ParmBuffer, fp );
            sprintf( ParmBuffer, "       CLWLLEN(%ld) +\n",
                                pQMgrDefn->WorkLoadLength);
            fputs( ParmBuffer, fp );

           } /* end of if > 510 */
       #endif

#if defined (MQCMD_INQUIRE_AUTH_INFO)
      if (UserVersion>=MQCMDL_LEVEL_530)
         {
            MQsprintf( ParmBuffer
                     , "       SSLCRLNL ('%s') +\n"
                     , pQMgrDefn->SslCrlNameList
                     , MQ_OBJECT_NAME_LENGTH
                     );
            fputs( ParmBuffer, fp );
	    /**                                **/
	    /** v530a, fixed length of SSLKEYR **/
	    /**                                **/
            MQsprintf( ParmBuffer
                     , "       SSLKEYR ('%s') +\n"
                     , pQMgrDefn->SslKeyRepos
                     , MQ_SSL_KEY_REPOSITORY_LENGTH
                     );
            fputs( ParmBuffer, fp );
            /* HW Crypto is only supported on Unix */
            if (MQAT_DEFAULT == MQAT_UNIX ) {
              MQsprintf( ParmBuffer
                         , "       SSLCRYP ('%s') +\n"
                         , pQMgrDefn->SslCryptoHW
                         , MQ_SSL_CRYPTO_HARDWARE_LENGTH
                         );
              fputs( ParmBuffer, fp );
            }

         } /* end of if > 530 */
#endif

      sprintf( ParmBuffer, "       FORCE\n" );
      fputs( ParmBuffer, fp );

      sprintf( ParmBuffer, "\n" );
      fputs( ParmBuffer, fp );

      /* v51 removes close from here ...fclose(fp); */

      free (ParmBuffer);
   return;

}


