/*===========================================================================*/
/*              This supportpac is maintained by Wayne M. Schutz             */
/*                                                                           */
/*              Chris Petty is the original author.                          */
/*                                                                           */
/*  (C) Copyright IBM Corporation 1995, 1997, 1998, 1999, 2000, 2002, 2003   */
/*===========================================================================*/
/*                                                                           */
/*  See the read.me file for change history                                  */
/*                                                                           */
/*===========================================================================*/
/* Module Name: queue.c                                                      */
/*===========================================================================*/

/* Include standard libraries */
#include <stdlib.h>
#include <string.h>
#include <stdio.h>

/* Include MQSeries headers */
#include <cmqc.h>
#include <cmqcfc.h>
#include <cmqxc.h>

/* Include Application headers */
#include "saveqmgr.h"



/* ------------------------------------------------------------------------ */
/* ProcessQueueStringParm()                                                 */
/* ProcessQueueIntegerParm()                                                */
/*                                                                          */
/* These two processes are given a pointer to a block of storage that       */
/* contains the definition of a single attribute relating to an individual  */
/* queue. The first handles string parameters, the second integer.          */
/* The calling process has already decided which type of parameter this is  */
/* and is responsible for managing the storage. These procedures decide     */
/* which of the possible attributes this one is and extract the value.      */
/*                                                                          */
/* Follows is a list of all attributes possible for the complete            */
/* set of queue types; namely QLOCAL, QALIAS, QREMOTE and QMODEL.           */
/*                                                                          */
/* The parameters are not all relevant for all queue types.                 */
/*                                                                          */
/* The QDefn structure contains entries for each of the attributes,         */
/* only the attributes relevant for this queue type will be meaningful.     */
/* When we later write the file we will ensure that, based on queue type,   */
/* only the relevant fields are used.                                       */
/* ------------------------------------------------------------------------ */
extern int noisy;
extern int permDyn; /* v530 */   /* whether to create qlocal from Permdyn */

void ProcessQueueStringParm( MQCFST *pPCFString
                           , QParms *QDefn
                           , MQLONG    *iCC
                           )
{
   /* Initialise return codes */
   *iCC = OK;

   switch ( pPCFString->Parameter ) {
   case MQCA_Q_NAME:
      memset( QDefn->QName
            , ' ', MQ_Q_NAME_LENGTH );
      memcpy( QDefn->QName
            , pPCFString->String, pPCFString->StringLength);
      break;
   case MQCA_Q_DESC:
      memset( QDefn->QDesc
            , ' ', MQ_Q_DESC_LENGTH );
      memcpy( QDefn->QDesc
            , pPCFString->String, pPCFString->StringLength);
      break;
   case MQCA_PROCESS_NAME:
      memset( QDefn->ProcessName
            , ' ', MQ_PROCESS_NAME_LENGTH );
      memcpy( QDefn->ProcessName
            , pPCFString->String, pPCFString->StringLength);
      break;
   case MQCA_BACKOUT_REQ_Q_NAME:
      memset( QDefn->BackoutReqQName
            , ' ', MQ_Q_NAME_LENGTH );
      memcpy( QDefn->BackoutReqQName
            , pPCFString->String, pPCFString->StringLength);
      break;
   case MQCA_CREATION_DATE:
      memset( QDefn->CreationDate
            , ' ', MQ_CREATION_DATE_LENGTH );
      memcpy( QDefn->CreationDate
            , pPCFString->String, pPCFString->StringLength);
      break;
   case MQCA_CREATION_TIME:
      memset( QDefn->CreationTime
            , ' ', MQ_CREATION_TIME_LENGTH );
      memcpy( QDefn->CreationTime
            , pPCFString->String, pPCFString->StringLength);
      break;
   case MQCA_INITIATION_Q_NAME:
      memset( QDefn->InitiationQName
            , ' ', MQ_Q_NAME_LENGTH );
      memcpy( QDefn->InitiationQName
            , pPCFString->String, pPCFString->StringLength);
      break;
   case MQCA_TRIGGER_DATA:
      memset( QDefn->TriggerData
            , ' ', MQ_TRIGGER_DATA_LENGTH );
      memcpy( QDefn->TriggerData
            , pPCFString->String, pPCFString->StringLength);
      break;
   case MQCA_BASE_Q_NAME:
      memset( QDefn->BaseQName
            , ' ', MQ_Q_NAME_LENGTH );
      memcpy( QDefn->BaseQName
            , pPCFString->String, pPCFString->StringLength);
      break;
   case MQCA_REMOTE_Q_NAME:
      memset( QDefn->RemoteQName
            , ' ', MQ_Q_NAME_LENGTH );
      memcpy( QDefn->RemoteQName
            , pPCFString->String, pPCFString->StringLength);
      break;
   case MQCA_REMOTE_Q_MGR_NAME:
      memset( QDefn->RemoteQMgrName
            , ' ', MQ_Q_MGR_NAME_LENGTH );
      memcpy( QDefn->RemoteQMgrName
            , pPCFString->String, pPCFString->StringLength);
      break;
   case MQCA_XMIT_Q_NAME:
      memset( QDefn->XmitQName
            , ' ', MQ_Q_NAME_LENGTH );
      memcpy( QDefn->XmitQName
            , pPCFString->String, pPCFString->StringLength);
      break;
/* [v51] begin v51 attributes */
#if defined (MQCA_CLUSTER_NAME)
   case MQCA_CLUSTER_NAME:
      memset( QDefn->ClusterName
            , ' ', MQ_CLUSTER_NAME_LENGTH );
      memcpy( QDefn->ClusterName
            , pPCFString->String, pPCFString->StringLength);
      break;
#endif
#if defined (MQCA_CLUSTER_NAMELIST)
   case MQCA_CLUSTER_NAMELIST:
      memset( QDefn->ClusterNameList
            , ' ', MQ_OBJECT_NAME_LENGTH );
      memcpy( QDefn->ClusterNameList
            , pPCFString->String, pPCFString->StringLength);
      break;
#endif
#if defined (MQCA_ALTERATION_DATE)
   case MQCA_ALTERATION_DATE:
      memset( QDefn->AlterationDate
            , ' ', MQ_CREATION_DATE_LENGTH );
      memcpy( QDefn->AlterationDate
            , pPCFString->String, pPCFString->StringLength);
      break;
   case MQCA_ALTERATION_TIME:
      memset( QDefn->AlterationTime
            , ' ', MQ_CREATION_TIME_LENGTH );
      memcpy( QDefn->AlterationTime
            , pPCFString->String, pPCFString->StringLength);
      break;
#endif
/* [v51] end V51 attributes */
   default:
       if(noisy) fprintf(stderr, "Unknown PCF String Parameter: %ld\n",
              pPCFString->Parameter);
      *iCC = WARNING;
      break;
   } /* endswitch */

   return;
}



void ProcessQueueIntegerParm( MQCFIN *pPCFInteger
                            , QParms *QDefn
                            , MQLONG    *iCC
                            )
{
   /* Initialise return codes */
   *iCC = OK;                        /*[wms] fix */

   switch ( pPCFInteger->Parameter ) {
   case MQIA_Q_TYPE:
      QDefn->QType = pPCFInteger->Value;
      break;
   case MQIA_INHIBIT_PUT:
      QDefn->InhibitPut = pPCFInteger->Value;
      break;
   case MQIA_DEF_PRIORITY:
      QDefn->DefPriority = pPCFInteger->Value;
      break;
   case MQIA_DEF_PERSISTENCE:
      QDefn->DefPersistence = pPCFInteger->Value;
      break;
   case MQIA_INHIBIT_GET:
      QDefn->InhibitGet = pPCFInteger->Value;
      break;
   case MQIA_SCOPE:
      QDefn->Scope = pPCFInteger->Value;
      break;
   case MQIA_MAX_Q_DEPTH:
      QDefn->MaxQDepth = pPCFInteger->Value;
      break;
   case MQIA_MAX_MSG_LENGTH:
      QDefn->MaxMsgLength = pPCFInteger->Value;
      break;
   case MQIA_BACKOUT_THRESHOLD:
      QDefn->BackoutThreshold = pPCFInteger->Value;
      break;
   case MQIA_SHAREABILITY:
      QDefn->Shareability = pPCFInteger->Value;
      break;
   case MQIA_DEF_INPUT_OPEN_OPTION:
      QDefn->DefInputOpenOption = pPCFInteger->Value;
      break;
   case MQIA_HARDEN_GET_BACKOUT:
      QDefn->HardenGetBackout = pPCFInteger->Value;
      break;
   case MQIA_MSG_DELIVERY_SEQUENCE:
      QDefn->MsgDeliverySequence = pPCFInteger->Value;  /* [wms] fix only */
      break;
   case MQIA_RETENTION_INTERVAL:
      QDefn->RetentionInterval = pPCFInteger->Value;
      break;
   case MQIA_DEFINITION_TYPE:
      QDefn->DefinitionType = pPCFInteger->Value;
      break;
   case MQIA_USAGE:
      QDefn->Usage = pPCFInteger->Value;
      break;
   case MQIA_OPEN_INPUT_COUNT:
      QDefn->OpenInputCount = pPCFInteger->Value;
      break;
   case MQIA_OPEN_OUTPUT_COUNT:
      QDefn->OpenOutputCount = pPCFInteger->Value;
      break;
   case MQIA_CURRENT_Q_DEPTH:
      QDefn->CurrentQDepth = pPCFInteger->Value;
      break;
   case MQIA_TRIGGER_CONTROL:
      QDefn->TriggerControl = pPCFInteger->Value;
      break;
   case MQIA_TRIGGER_TYPE:
      QDefn->TriggerType = pPCFInteger->Value;
      break;
   case MQIA_TRIGGER_MSG_PRIORITY:
      QDefn->TriggerMsgPriority = pPCFInteger->Value;
      break;
   case MQIA_TRIGGER_DEPTH:
      QDefn->TriggerDepth = pPCFInteger->Value;
      break;
   case MQIA_Q_DEPTH_HIGH_LIMIT:
      QDefn->QDepthHighLimit = pPCFInteger->Value;
      break;
   case MQIA_Q_DEPTH_LOW_LIMIT:
      QDefn->QDepthLowLimit = pPCFInteger->Value;
      break;
   case MQIA_Q_DEPTH_MAX_EVENT:
      QDefn->QDepthMaxEvent = pPCFInteger->Value;
      break;
   case MQIA_Q_DEPTH_HIGH_EVENT:
      QDefn->QDepthHighEvent = pPCFInteger->Value;
      break;
   case MQIA_Q_DEPTH_LOW_EVENT:
      QDefn->QDepthLowEvent = pPCFInteger->Value;
      break;
   case MQIA_Q_SERVICE_INTERVAL:
      QDefn->QServiceInterval = pPCFInteger->Value;
      break;
   case MQIA_Q_SERVICE_INTERVAL_EVENT:
      QDefn->QServiceIntervalEvent = pPCFInteger->Value;
      break;
#if defined (MQIA_DIST_LISTS)
   /* [wms] start V5 attributes */
   case MQIA_DIST_LISTS:
      QDefn->DistLists = pPCFInteger->Value;
      break;
   /* [wms] end V5 attributes */
#endif
#if defined (MQIA_DEF_BIND)
   /* [v51] start V51 attributes */
   case MQIA_DEF_BIND:
      QDefn->DefBind = pPCFInteger->Value;
      break;
   /* [v51] end V51 attributes */
#endif
   default:
       if(noisy) fprintf(stderr, "Unknown PCF Integer Parameter: %ld\n",
              pPCFInteger->Parameter);
      *iCC = WARNING;
      break;
   } /* endswitch */

   return;
}



/* ------------------------------------------------------------------------- */
/*                                                                           */
/* This process takes the attributes of a single queue and adds them         */
/* to the end of a file, SAVEQMGR.TST, which can be found in the current     */
/* directory.                                                                */
/*                                                                           */
/* The file is of a format suitable for subsequent input to RUNMQSC.         */
/*                                                                           */
/* ------------------------------------------------------------------------- */

void AddToFileQUEUE(
                     QParms * pQDefn,
                     int UserVersion,
                     char * fileName,          /* v32 Name of the output file  */
                        FILE * fp,              /* v51 */
                     MQLONG *iCC
                     )
{
   char    *ParmBuffer;   /* Temporary buffer to hold for output to file */

   /* Initialise return codes */
   *iCC = OK;

   if ( !( ParmBuffer = (char *) malloc (PBSIZE) )) {
     fprintf (stderr, "Failed to allocate %d bytes \n", PBSIZE);
     exit(2);
   }

   /* [v530]  Process  all local queues and any predefined queues */
   /* [v530] If the user so indicates, also process PermDyn queues */

 if (    (pQDefn->QType!=MQQT_LOCAL) ||    /* Process all non-local queues */
         (pQDefn->DefinitionType == MQQDT_PREDEFINED) ||
         permDyn && (pQDefn->DefinitionType == MQQDT_PERMANENT_DYNAMIC) )
    {

   /* Append these details to the end of the current output file (v32) */
   /* v51 remove open from here ...   fp = fopen( fileName, OPENMODE ); */


      /* ---------------------------------------------------------------- */
      /* Parameters general to all queue types                            */
      /* ---------------------------------------------------------------- */

      switch ( pQDefn->QType ) {
      case MQQT_LOCAL:
         MQsprintf( ParmBuffer
                  , "DEFINE QLOCAL ('%s') +\n"
                  , pQDefn->QName
                  , MQ_Q_NAME_LENGTH
                  );
         break;
      case MQQT_ALIAS:
         MQsprintf( ParmBuffer
                  , "DEFINE QALIAS ('%s') +\n"
                  , pQDefn->QName
                  , MQ_Q_NAME_LENGTH
                  );
         break;
      case MQQT_MODEL:
         MQsprintf( ParmBuffer
                  , "DEFINE QMODEL ('%s') +\n"
                  , pQDefn->QName
                  , MQ_Q_NAME_LENGTH
                  );
         break;
      case MQQT_REMOTE:
         MQsprintf( ParmBuffer
                  , "DEFINE QREMOTE ('%s') +\n"
                  , pQDefn->QName
                  , MQ_Q_NAME_LENGTH
                  );
         break;
      default:
         *iCC = WARNING;
         if(noisy) fprintf(stderr, "Unknown queue type = %ld \n", pQDefn->QType);
         break;
      } /* endswitch */
      fputs( ParmBuffer, fp );

      if (  pQDefn->QType == MQQT_LOCAL
            || pQDefn->QType == MQQT_MODEL) {
        MQsprintf( ParmBuffer, "* CRDATE (%s) + \n",
                   pQDefn->CreationDate,
                   MQ_CREATION_DATE_LENGTH);
        fputs( ParmBuffer, fp);
        MQsprintf( ParmBuffer, "* CRTIME (%s) + \n",
                   pQDefn->CreationTime,
                   MQ_CREATION_TIME_LENGTH);
        fputs( ParmBuffer, fp);
      }

      #if defined (MQCA_ALTERATION_DATE)
      MQsprintf( ParmBuffer, "* ALTDATE (%s) + \n",
               pQDefn->AlterationDate,
               MQ_CREATION_DATE_LENGTH);
      fputs( ParmBuffer, fp);
      MQsprintf( ParmBuffer, "* ALTTIME (%s) + \n",
               pQDefn->AlterationTime,
               MQ_CREATION_TIME_LENGTH);
      fputs( ParmBuffer, fp);
      #endif

      /* [v530] If we're making a local q from a Permdyn one, */
      /* [v530] then make a little note in the descr field */

      if (permDyn &&
          pQDefn->QType == MQQT_LOCAL &&
          pQDefn->DefinitionType == MQQDT_PERMANENT_DYNAMIC )
        {
           MQsprintf( ParmBuffer
                    , "       DESCR('(Defn from PermDyn Q)%s') +\n"
                    , pQDefn->QDesc
                    , MQ_Q_DESC_LENGTH - 21
                    );
        }
      else
        {
           MQsprintf( ParmBuffer
                    , "       DESCR('%s') +\n"
                    , pQDefn->QDesc
                    , MQ_Q_DESC_LENGTH
                    );
        }/* End if*/
      fputs( ParmBuffer, fp );

      if ( pQDefn->InhibitPut == MQQA_PUT_ALLOWED ) {
         sprintf( ParmBuffer, "       PUT(ENABLED) +\n" );
         fputs( ParmBuffer, fp );
      } else {
         sprintf( ParmBuffer, "       PUT(DISABLED) +\n" );
         fputs( ParmBuffer, fp );
      } /* endif */

      sprintf( ParmBuffer, "       DEFPRTY(%ld) +\n", pQDefn->DefPriority );
      fputs( ParmBuffer, fp );

      if ( pQDefn->DefPersistence == MQPER_PERSISTENT ) {
         sprintf( ParmBuffer, "       DEFPSIST(YES) +\n" );
         fputs( ParmBuffer, fp );
      } else {
         sprintf( ParmBuffer, "       DEFPSIST(NO) +\n" );
         fputs( ParmBuffer, fp );
      } /* endif */



      /* ---------------------------------------------------------------- */
      /* Parameters common to QLOCAL, QALIAS and QREMOTE                  */
      /* ---------------------------------------------------------------- */
      if (  MQAT_DEFAULT != MQAT_OS400 &&           /* v510 AvV */
            UserVersion > MQCMDL_LEVEL_110 && (
             pQDefn->QType == MQQT_LOCAL
         || pQDefn->QType == MQQT_ALIAS
         || pQDefn->QType == MQQT_REMOTE )) {
               if ( pQDefn->Scope == MQSCO_Q_MGR ) {
                  sprintf( ParmBuffer, "       SCOPE(QMGR) +\n" );
                  fputs( ParmBuffer, fp );
               } else {
                  sprintf( ParmBuffer, "       SCOPE(CELL) +\n" );
                  fputs( ParmBuffer, fp );
               } /* endif */

      } /* endif */


      /* ---------------------------------------------------------------- */
      /* Parameters common to QLOCAL, QALIAS and QMODEL                   */
      /* ---------------------------------------------------------------- */
      if (  pQDefn->QType == MQQT_LOCAL
         || pQDefn->QType == MQQT_ALIAS
         || pQDefn->QType == MQQT_MODEL
         ) {

         if ( pQDefn->InhibitGet == MQQA_GET_ALLOWED ) {
            sprintf( ParmBuffer, "       GET(ENABLED) +\n" );
            fputs( ParmBuffer, fp );
         } else {
            sprintf( ParmBuffer, "       GET(DISABLED) +\n" );
            fputs( ParmBuffer, fp );
         } /* endif */

      } /* endif */



      /* ---------------------------------------------------------------- */
      /* Parameters exclusive to QALIAS                                   */
      /* ---------------------------------------------------------------- */
      if ( pQDefn->QType == MQQT_ALIAS ) {

         MQsprintf( ParmBuffer
                  , "       TARGQ('%s') +\n"
                  , pQDefn->BaseQName
                  , MQ_Q_NAME_LENGTH
                  );
         fputs( ParmBuffer, fp );

      } /* endif */



      /* ---------------------------------------------------------------- */
      /* Parameters exclusive to QREMOTE                                  */
      /* ---------------------------------------------------------------- */
      if ( pQDefn->QType == MQQT_REMOTE ) {

         MQsprintf( ParmBuffer
                  , "       XMITQ('%s') +\n"
                  , pQDefn->XmitQName
                  , MQ_Q_NAME_LENGTH
                  );
         fputs( ParmBuffer, fp );

         MQsprintf( ParmBuffer
                  , "       RNAME('%s') +\n"
                  , pQDefn->RemoteQName
                  , MQ_Q_NAME_LENGTH
                  );
         fputs( ParmBuffer, fp );

         MQsprintf( ParmBuffer
                  , "       RQMNAME('%s') +\n"
                  , pQDefn->RemoteQMgrName
                  , MQ_Q_MGR_NAME_LENGTH
                  );
         fputs( ParmBuffer, fp );

      } /* endif */



      /* ---------------------------------------------------------------- */
      /* Parameters exclusive to QMODEL                                   */
      /* ---------------------------------------------------------------- */
      if ( pQDefn->QType == MQQT_MODEL ) {

         switch ( pQDefn->DefinitionType ) {
         case MQQDT_PREDEFINED:
            break;
         case MQQDT_PERMANENT_DYNAMIC:
            sprintf( ParmBuffer, "       DEFTYPE(PERMDYN) +\n" );
            fputs( ParmBuffer, fp );
            break;
         case MQQDT_TEMPORARY_DYNAMIC:
            sprintf( ParmBuffer, "       DEFTYPE(TEMPDYN) +\n" );
            fputs( ParmBuffer, fp );
            break;
         } /* endswitch */

      } /* endif */



      /* ---------------------------------------------------------------- */
      /* Parameters common to QLOCAL and QMODEL                           */
      /* ---------------------------------------------------------------- */
      if (  pQDefn->QType == MQQT_LOCAL
         || pQDefn->QType == MQQT_MODEL
         ) {

         /* [v530] here's a little hack, if we're generating pre 5.3 */
         /* mqsc, maxdepth can't be greater than 640000 */
         if (UserVersion < MQCMDL_LEVEL_530 &&
              pQDefn->MaxQDepth > 640000 )
              pQDefn->MaxQDepth = 640000;


         sprintf( ParmBuffer, "       MAXDEPTH(%ld) +\n", pQDefn->MaxQDepth );
         fputs( ParmBuffer, fp );

         sprintf( ParmBuffer, "       MAXMSGL(%ld) +\n", pQDefn->MaxMsgLength );
         fputs( ParmBuffer, fp );

         if ( pQDefn->Shareability == MQQA_SHAREABLE ) {
            sprintf( ParmBuffer, "       SHARE +\n" );
            fputs( ParmBuffer, fp );
         } else {
            sprintf( ParmBuffer, "       NOSHARE +\n" );
            fputs( ParmBuffer, fp );
         } /* endif */

         if ( pQDefn->DefInputOpenOption == MQOO_INPUT_SHARED ) {
            sprintf( ParmBuffer, "       DEFSOPT(SHARED) +\n" );
            fputs( ParmBuffer, fp );
         } else {
            sprintf( ParmBuffer, "       DEFSOPT(EXCL) +\n" );
            fputs( ParmBuffer, fp );
         } /* endif */

         if ( pQDefn->MsgDeliverySequence == MQMDS_PRIORITY ) {
            sprintf( ParmBuffer, "       MSGDLVSQ(PRIORITY) +\n" );
            fputs( ParmBuffer, fp );
         } else {
            sprintf( ParmBuffer, "       MSGDLVSQ(FIFO) +\n" );
            fputs( ParmBuffer, fp );
         } /* endif */

         if ( pQDefn->HardenGetBackout == MQQA_BACKOUT_HARDENED ) {
            sprintf( ParmBuffer, "       HARDENBO +\n" );
            fputs( ParmBuffer, fp );
         } else {
            sprintf( ParmBuffer, "       NOHARDENBO +\n" );
            fputs( ParmBuffer, fp );
         } /* endif */

         if ( pQDefn->Usage == MQUS_NORMAL ) {
            sprintf( ParmBuffer, "       USAGE(NORMAL) +\n" );
            fputs( ParmBuffer, fp );
         } else {
            sprintf( ParmBuffer, "       USAGE(XMITQ) +\n" );
            fputs( ParmBuffer, fp );
         } /* endif */

         /* v32 mqseries for windows is always NOtrigger */
         if ( UserVersion > MQCMDL_LEVEL_110 ) {

         if (pQDefn->TriggerControl == MQTC_OFF ) {
            sprintf( ParmBuffer, "       NOTRIGGER +\n" );
            fputs( ParmBuffer, fp );
         } else {
            sprintf( ParmBuffer, "       TRIGGER +\n" );
            fputs( ParmBuffer, fp );
         } /* endif */

         switch ( pQDefn->TriggerType ) {
         case MQTT_NONE:
            sprintf( ParmBuffer, "       TRIGTYPE(NONE) +\n" );
            fputs( ParmBuffer, fp );
            break;
         case MQTT_FIRST:
            sprintf( ParmBuffer, "       TRIGTYPE(FIRST) +\n" );
            fputs( ParmBuffer, fp );
            break;
         case MQTT_EVERY:
            sprintf( ParmBuffer, "       TRIGTYPE(EVERY) +\n" );
            fputs( ParmBuffer, fp );
            break;
         case MQTT_DEPTH:
            sprintf( ParmBuffer, "       TRIGTYPE(DEPTH) +\n" );
            fputs( ParmBuffer, fp );
            break;
         } /* endswitch */

         sprintf( ParmBuffer, "       TRIGDPTH(%ld) +\n", pQDefn->TriggerDepth );
         fputs( ParmBuffer, fp );

         sprintf( ParmBuffer, "       TRIGMPRI(%ld) +\n", pQDefn->TriggerMsgPriority);
         fputs( ParmBuffer, fp );

         MQsprintf( ParmBuffer
                  , "       TRIGDATA('%s') +\n"
                  , pQDefn->TriggerData
                  , MQ_TRIGGER_DATA_LENGTH
                  );
         fputs( ParmBuffer, fp );

         MQsprintf( ParmBuffer
                  , "       PROCESS('%s') +\n"
                  , pQDefn->ProcessName
                  , MQ_PROCESS_NAME_LENGTH
                  );
         fputs( ParmBuffer, fp );

         MQsprintf( ParmBuffer
                  , "       INITQ('%s') +\n"
                  , pQDefn->InitiationQName
                  , MQ_Q_NAME_LENGTH
                  );
         fputs( ParmBuffer, fp );

         } /* v32 end of not supported by mq/win */


         sprintf( ParmBuffer, "       RETINTVL(%ld) +\n", pQDefn->RetentionInterval );
         fputs( ParmBuffer, fp );

         sprintf( ParmBuffer, "       BOTHRESH(%ld) +\n", pQDefn->BackoutThreshold );
         fputs( ParmBuffer, fp );

         MQsprintf( ParmBuffer
                 , "       BOQNAME('%s') +\n"
                 , pQDefn->BackoutReqQName
                 , MQ_Q_NAME_LENGTH
                 );
         fputs( ParmBuffer, fp );

         sprintf( ParmBuffer, "       QDEPTHHI(%ld) +\n", pQDefn->QDepthHighLimit );
         fputs( ParmBuffer, fp );

         sprintf( ParmBuffer, "       QDEPTHLO(%ld) +\n", pQDefn->QDepthLowLimit );
         fputs( ParmBuffer, fp );

         if ( pQDefn->QDepthMaxEvent == MQEVR_ENABLED ) {
            sprintf( ParmBuffer, "       QDPMAXEV(ENABLED) +\n" );
            fputs( ParmBuffer, fp );
         } else {
            sprintf( ParmBuffer, "       QDPMAXEV(DISABLED) +\n" );
            fputs( ParmBuffer, fp );
         } /* endif */

         if ( pQDefn->QDepthHighEvent == MQEVR_ENABLED ) {
            sprintf( ParmBuffer, "       QDPHIEV(ENABLED) +\n" );
            fputs( ParmBuffer, fp );
         } else {
            sprintf( ParmBuffer, "       QDPHIEV(DISABLED) +\n" );
            fputs( ParmBuffer, fp );
         } /* endif */

         if ( pQDefn->QDepthLowEvent == MQEVR_ENABLED ) {
            sprintf( ParmBuffer, "       QDPLOEV(ENABLED) +\n" );
            fputs( ParmBuffer, fp );
         } else {
            sprintf( ParmBuffer, "       QDPLOEV(DISABLED) +\n" );
            fputs( ParmBuffer, fp );
         } /* endif */

         sprintf( ParmBuffer, "       QSVCINT(%ld) +\n", pQDefn->QServiceInterval );
         fputs( ParmBuffer, fp );

         switch ( pQDefn->QServiceIntervalEvent ) {
         case MQQSIE_OK:
            sprintf( ParmBuffer, "       QSVCIEV(OK) +\n" );
            fputs( ParmBuffer, fp );
            break;
         case MQQSIE_NONE:
            sprintf( ParmBuffer, "       QSVCIEV(NONE) +\n" );
            fputs( ParmBuffer, fp );
            break;
         case MQQSIE_HIGH:
            sprintf( ParmBuffer, "       QSVCIEV(HIGH) +\n" );
            fputs( ParmBuffer, fp );
            break;
         } /* endswitch */
#if defined(MQDL_SUPPORTED)
         /* [wms] start V5 attributes */

         if (UserVersion>=MQCMDL_LEVEL_500)
           {
              if ( pQDefn->DistLists == MQDL_SUPPORTED ) {
                 sprintf( ParmBuffer, "       DISTL(YES) +\n" );
                 fputs( ParmBuffer, fp );
              } else {
                 sprintf( ParmBuffer, "       DISTL(NO) +\n" );
                 fputs( ParmBuffer, fp );
              } /* endif */

           }
         /* [wms]  end V5 attributes */
#endif

      } /* endif */

      /* ---------------------------------------------------------------- */
      /* [v5] Parms common to QLOCAL and QREMOTE and QALIAS               */
      /* ---------------------------------------------------------------- */
      if (  pQDefn->QType == MQQT_LOCAL
         || pQDefn->QType == MQQT_REMOTE
         || pQDefn->QType == MQQT_ALIAS
         ) {

#if defined(MQ_CLUSTER_NAME_LENGTH)
         /* [v51] start V51 attributes */

         if (UserVersion>=MQCMDL_LEVEL_510)
           {

         MQsprintf( ParmBuffer
                  , "       CLUSTER('%s') +\n"
                  , pQDefn->ClusterName
                  , MQ_CLUSTER_NAME_LENGTH
                  );
         fputs( ParmBuffer, fp );
         MQsprintf( ParmBuffer
                  , "       CLUSNL('%s') +\n"
                  , pQDefn->ClusterNameList
                  , MQ_CLUSTER_NAME_LENGTH
                  );
         fputs( ParmBuffer, fp );
         if ( pQDefn->DefBind == MQBND_BIND_ON_OPEN ) {
            sprintf( ParmBuffer, "       DEFBIND(OPEN) +\n" );
            fputs( ParmBuffer, fp );
         } else {
            sprintf( ParmBuffer, "       DEFBIND(NOTFIXED) +\n" );
            fputs( ParmBuffer, fp );
         } /* endif */

        } /* end of user want v51 */
#endif

      } /* end of local remote and alias common defs */


      sprintf( ParmBuffer, "       REPLACE\n" );
      fputs( ParmBuffer, fp );

      sprintf( ParmBuffer, "\n" );
      fputs( ParmBuffer, fp );

     /* remove close from here ... fclose(fp); */

    }
  else  /* Its a dynmaic queue and we don't want to process it */
    {
      fprintf(stderr, "Skipping dynamic queue %.48s\n", pQDefn->QName);
   } /* endif */

 free (ParmBuffer);
   return;

}


